<?php
$marks = 0;
$fullMarks = 5;
$solutionFile = "../../../../../solutions/q14/Olympics.php";
$answerFile = "./Olympics.php";

$info = $warn = $error = [];

// Check if the submission is identical to the solution
$solution = strip($solutionFile);
$answer = strip($answerFile);
if ($solution === $answer) {
    $marks = $fullMarks;
    $info[] = "Answer is identical to the solution.&nbsp;&nbsp;&nbsp;=> Full Marks: $marks.<br>
    No further checks needed!";
    return;
}

// class and methods
$class = "Olympics";
$properties = ["year", "country", "sports"];

// To standardize parameters to the names in the solution file
$methods = ["checkSport"];
$replace = $properties;
$replace[] = 'name_of_sport';
$answer = standardize($answerFile, $class, $replace, $methods);
if ($solution === $answer) {
    // May still have errors in the property declarations
    $warn[] = "Answer (standardized) is identical to the solution. <br>
    Maybe Full Marks: $marks. <br>
    Still Checking...";
}

// Test case hardcoded
$year = 2024;
$country = "France";
$sports = ["badminton", "chess"];

try {
    require_once $answerFile;
    if (!class_exists($class, false)) { // false => no autoloading
        $error[] = "<code>$class</code> does not exist.";
    }
    $publicProperties = get_class_vars($class);
    foreach ($properties as $property) {
        if (property_exists($class, $property)) {
            if (array_key_exists($property, $publicProperties)) {
                $error[] = "Property <code>$property</code> is public.";
                $info[] = "Property <code>$property</code> is public &nbsp;&nbsp;&nbsp;=> 0.250 instead of 0.333";
                $marks += 0.25;
            } else {
                $info[] = "Property <code>$property</code> defined &nbsp;&nbsp;&nbsp;=> 0.333";
                $marks += 1 / 3;
            }
        } else {
            $error[] = "Property <code>$property</code> does not exist.";
        }
    }
    foreach ($properties as $property) {
        $getter = "get" . ucfirst($property);
        if (method_exists($class, $getter)) {
            $marks += 0.25; // 0.25 if exists, 0.25 if works
            $info[] = "Getter <code>$getter</code> defined &nbsp;&nbsp;&nbsp;=> 0.250";
        } else {
            $error[] = "Getter <code>$getter</code> does not exist.";
        }
    }
    $o = new $class($year, $country, $sports);
    // If constructed okay, __constructor is good
    $marks += 0.5;
    foreach ($properties as $property) {
        $getter = "get" . ucfirst($property);
        if (method_exists($class, $getter) && @$o->$getter() == $$property) {
            $marks += 0.25; // Getter works => 0.25 each
            $info[] = "Getter <code>$getter</code> works &nbsp;&nbsp;&nbsp;=> 0.250";
            $marks += 0.5 / 3; // also means constructor initialized properly
            $info[] = "Constructor assigns Property <code>$property</code> defined => 0.167";
        } else {
            $error[] = "Getter <code>$getter()</code> does not work.";
        }
    }
    foreach ($methods as $method) {
        if (method_exists($class, $method)) {
            $sport = "chess";
            $returned = @$o->$method($sport);
            // No invocation error means $method takes one arg
            $marks += 0.5;
            $info[] = "Method <code>$method</code> is defined and takes one sport parameter.&nbsp;&nbsp;&nbsp;=> 0.500";
            if ($returned == $year) {
                $marks += 0.5;
                $info[] = "Method <code>$method</code> returns years if sport exists.";
            } else {
                $error[] = "Method <code>$method</code> does not return year: $returned != $year.";
            }
            $sport = "tennis";
            $returned = @$o->$method($sport);
            if ($returned === false) {
                $marks += 0.5;
                $info[] = "Method <code>$method</code> returns false when needed.&nbsp;&nbsp;&nbsp;=> 0.500";
            }else {
                $error[] = "Method <code>$method</code> does not return false when needed: Returns $returned.";
            }
        } else {
            $error[] = "Method <code>$method</code> does not exist.";
        }
    }
} catch (Throwable $e) {
    echo "<h1 style='color:red'>Exceptions Caught: Possible Coding Errors</h1>";
    echo "<pre style='color:red;font-weight:bold'>{$e->getMessage()}</pre>";
    echo "<p>May need to edit the following file before continuing:</p>";
    $realPath = realpath($answerFile);
    echo "<p style='background-color:yellow;color:red;font-weight:bold'>
    <code>$realPath</code></p>";
}
if (count(array_merge($info, $warn, $error))) {
    echo "<h2  style='color:green'>More Information</h2>";
    say($info, "Marking Details", "green");
    say($warn, "Warnings", "blue");
    say($error, "Some Tests Failed", "red");
}

$marks = number_format($marks, 3);
echo "<h2>Total Marks out of $fullMarks</h2>
<h3>q14: $marks</h3>";

function say($what, $title, $color = "black")
{
    if (!empty($what)) {
        echo "<h3 style='color:$color'>$title</h3>
            <h4 style='color:$color'> Details:</h4> 
            <ul style='color:$color'><li>" .
            implode("</li><li>", $what) . "</li></ul>";
    }
}

function standardize($answerFile, $class, $replace, $methods)
{
    require_once $answerFile;
    $reflected = new ReflectionClass($class);
    $parameters = $reflected->getConstructor()->getParameters();
    $search = [];
    foreach ($parameters as $parameter) {
        $search[] = '$' . $parameter->getName();
    }
    foreach ($methods as $method) {
        $parameters = $reflected->getMethod($method)->getParameters();
        foreach ($parameters as $parameter) {
            $search[] = '$' . $parameter->getName();
        }
    }
    $replace = array_map(function ($s) {
        return '$' . $s;
    }, $replace);
    $answer = file_get_contents($answerFile);
    $answerStd = str_replace($search, $replace, $answer);
    $answerFileStd = str_replace(".php", "-std.php", $answerFile);
    file_put_contents($answerFileStd, $answerStd);
    return $answerFileStd;
}

function strip($file)
{
    $stripped = php_strip_whitespace($file);
    $stripped = str_replace(["<?php", "?>", " "], "", $stripped);
    return $stripped;
}