<?php
/**
 * Summary of CFG
 */
class CFG extends HT
{
  // ------------ Imported from config.php ------------
  // Exam details
  public static $examShortName;
  public static $examLongName;
  public static $mode = "LT"; // LT or FT, lower or uppercase
  public static $sectionNames; // Your sections
  private static $csvFileNamePattern; // Files where the eLearn Grade (or Quiz) exports are stored
  public static $locSubQ; // Location of subquestion number and marks in the CSV file header
  public static $locMarks;
  public static $rubricFile = "./rubric.csv"; // Rubric file
  private static $submissionFolderPattern; // Folders where the eLearn Assignments (student zip files) are stored
  public static $emailFolder; // Folders where the emailed and repaired zip files are stored
  private static $answerFolderPattern; // Folders where the zip files are unzipped
  public static $solutionFolder;   // Solutions
  public static $resourceFolder; //  Resources

  // ------------ End of imports from config.php ------------
  private static $vars = [
    "examShortName",
    "examLongName",
    "mode",
    "sectionNames",
    "csvFileNamePattern",
    "locSubQ",
    "locMarks",
    "rubricFile",
    "submissionFolderPattern",
    "answerFolderPattern",
    "emailFolder",
    "solutionFolder",
    "resourceFolder",
    "gradeQuestions",
    "autoGrade",
    "autoGradeOptions",
    "dbUser",
    "dbPass",
    "dbName"
  ];
  public static $configFile = "../config.php";
  private static $sqlTemplate = "../model/setup.template.sql";
  private static $sqlFile = 'setup.sql';
  private static $root;
  // Questions to grade (so that the instructors can divide the labor)
  public static $gradeQuestions;
  public static $autoGrade;
  public static $autoGradeOptions;
  protected static $class = __CLASS__;
  // MySQL credentials
  public static $dbUser, $dbPass, $dbName, $dbPrefix;
  // Do not modify below this line
  protected static $actions = [
    'view' => 'View Configuration',
    'validate' => 'Validate Config and Folders',
    'viewSql' => 'View SQL File',
    'runSql' => 'Setup Database',
    'dropDB' => 'Drop Database',
  ];
  protected static $actionProperties = [
    'view' => [
      'target' => 'left',
      'clear' => 'yes'
    ],
    'validate' => [
      'target' => 'right'
    ],
    'viewSql' => [
      'target' => 'left',
      'clear' => 'yes'
    ],
    'runSql' => [
      'target' => 'right'
    ],
    'dropDB' => [
      'class' => 'error',
      'target' => 'right'
    ]
  ];
  protected static $actionDesc = [
    'view' => 'View (not edit) the current settings in your <code>config.php</code> file.',
    'validate' => 'Validate your configuration in <code>config.php</code> file and check for the existence an contents of the folders specified.',
    'viewSql' => 'Examine the SQL file that will be run to set up your database. Do not edit this file. It is generated based on your <code>config.php</code> and a template',
    'runSql' => 'Run the generated SQL file to set up the database. If you see a red box about errors in the database, try this button. Clicking the button again is harmless: It does not reset the database.',
    'dropDB' => 'Drop all database tables and restart: This button will really reset the database. If you have graded some students, all the entered grades will really disappear.'
  ];
  protected static $intro = "<h4>Configuration Menu</h4>
  This subPage allows you to view and validate your configuration file.
  Validation means checking for the existence of the necessary folders and files.
  This is also the place where you will set up the database tables. If necessary,
  you can reset your DB also here. See the descriptions of what the buttons do
  on the right.";
  protected static $display = [
    "class" => "plain",
    "title" => "Setup and Configuration"
  ];
  // Number of questions, to be counted and set in the Exam constructor
  public static $numQuestions;
  public static function init()
  {
    $vars = self::$vars;
    foreach ($vars as $v) {
      if (isset($GLOBALS[$v])) { // Necessary for $mode = "ft"
        self::$$v = $GLOBALS[$v];
      }
    }
    self::$dbName = strtolower(self::$dbName);
    $dbHost = "localhost";
    $db = new DB($dbHost, self::$dbUser, self::$dbPass, self::$dbName);
    self::$dbPrefix = self::getDbPrefix();
    self::$configFile = realpath(self::$configFile);
    self::$root = $GLOBALS['root'];
  }
  public static function setNumQuestions($numQuestions)
  {
    self::$numQuestions = $numQuestions;
  }
  public static function getCsvFileName($sectionName, $studentName = "")
  {
    return str_replace(
      ["{sectionName}", "{studentName}"],
      [trim($sectionName), trim($studentName)],
      self::$csvFileNamePattern
    );
  }
  public static function getSubmissionFolder($sectionName, $studentName = "")
  {
    return str_replace(
      ["{sectionName}", "{studentName}"],
      [trim($sectionName), trim($studentName)],
      self::$submissionFolderPattern
    );
  }
  public static function getAnswerFolder($sectionName, $studentName = "")
  {
    return str_replace(
      ["{sectionName}", "{studentName}"],
      [trim($sectionName), trim($studentName)],
      self::$answerFolderPattern
    );
  }
  public static function isQuiz()
  {
    return trim(strtolower(self::$mode)) == "ft";
  }
  public static function getQDirPattern()
  {
    // Create a regular expression like "@/q[123]/..@
    $pattern = "@/[qQ][";
    for ($i = 1; $i <= self::$numQuestions; $i++) {
      $pattern .= $i;
    }
    $pattern .= "]/..@";
    return $pattern;
  }
  // Dispatch handlers
  public static function view()
  {
    $configFile = self::$configFile;
    $ext = pathinfo($configFile, PATHINFO_EXTENSION);
    $configFileContents = file_get_contents($configFile);
    try {
      $includedConfigFile = explode("require_once ", $configFileContents)[1];
      $includedConfigFile = explode('"', $includedConfigFile)[1];
      $includedConfigFile = str_replace('$root', self::$root, $includedConfigFile);
      $includedConfigFileContent = file_get_contents($includedConfigFile);
    } catch (Exception $e) {
      echo $e->getMessage();
    }
    $fullText0 = htmlentities($configFileContents);
    $fullText1 = htmlentities($includedConfigFileContent);
    $realPath0 = "<div 
      style='color: #373;
      background: #ecffd6;
      border: 1px solid #617c42;
      padding:4px;
      font-weight:bold;
      text-align:left;'><code>" .
      realpath($configFile) . "</code></div>";
    $realPath1 = "<div 
      style='color: #373;
      background: #ecffd6;
      border: 1px solid #617c42;
      padding:4px;
      font-weight:bold;
      text-align:left;'><code>" .
      realpath($includedConfigFile) . "</code></div>";
    HT::codeHead($configFile);
    echo "$realPath0<pre><code class='language-$ext'>$fullText0</code></pre>";
    echo "$realPath1<pre><code class='language-$ext'>$fullText1</code></pre>";
    HT::foot();
  }
  public static function validate()
  {
    HT::head();
    $error = false;
    $configFile = self::$configFile;
    $str = HT::info(
      "Validating <strong>" . self::$examShortName . ": " .
      self::$examLongName . "</strong><br>
      Config file: <a href='?do=view'><code>$configFile</code></a>",
      true
    );
    $root = self::$root;
    $realpath = realpath($root);
    if (!is_dir($root)) {
      $str .= HT::error("Exam folder (<code>$realpath</code>) not found!<br>
      Edit <code>$configFile</code> to set <code>\$root.</code>", true);
      $error = true;
      goto end;
    } else {
      $str .= HT::success("Exam folder (<code>$realpath</code>) located.", true);
    }
    if (self::isQuiz()) {
      $str .= HT::success(
        "Grading a finals (an eLearn quiz, as opposed to a Lab Test)!",
        true
      );
      self::$sectionNames = ['All'];
    }
    if (empty(self::$sectionNames)) {
      $str .= HT::error(
        "Section names (<code>\$sectionNames</code>) not specified!<br> 
        Edit <code>$configFile</code> to set it as an array.",
        true
      );
      $error = true;
      goto end;
    } else {
      $str1 = implode(", ", self::$sectionNames);
      $str .= HT::success(
        "Found these section[s]: <strong>$str1</strong>.",
        true
      );
      $str0 = "";
      foreach (self::$sectionNames as $sectionName) {
        $csvFileName = self::getCsvFileName($sectionName);
        if (is_file($csvFileName)) {
          $csvFileName = realpath($csvFileName);
          $str0 .= HT::success(
            "&nbsp;&rarr; eLearn Grade Export file located:<br>
            &emsp;<code>$csvFileName</code>",
            true
          );
        } else {
          $str0 .= HT::error(
            "&nbsp;&rarr; eLearn Grade Export file  not found.<br>
            &emsp;<code>$csvFileName</code><br>
            Please create it: Export the grade for the section $sectionName as CSV.",
            true
          );
          $error = true;
        }
      }
      if ($error) {
        $str .= HT::warn($str0, true);
      } else {
        $str .= HT::info($str0, true);
      }
      $str0 = "";
      foreach (self::$sectionNames as $sectionName) {
        $submissionFolder = self::getSubmissionFolder($sectionName);
        if (is_dir($submissionFolder)) {
          $submissionFolder = realpath($submissionFolder);
          $nZips = count(glob($submissionFolder . "/*.zip"));
          if ($nZips > 0) {
            $str0 .= HT::success(
              "&nbsp;&rarr; Found $nZips student submissions in<br>
              &emsp;<code>$submissionFolder</code>",
              true
            );
          } else {
            $str0 .= HT::warn(
              "&nbsp;&rarr; No zip files in the student submission folder
              &emsp;<code>$submissionFolder</code>",
              true
            );
            $error = true;
          }
        } else {
          $str0 .= HT::error(
            "&nbsp;&rarr; Student Submission folder not found:<br>
            &emsp;<code>$submissionFolder</code> <br>
            Please download the student submissions from $sectionName 
            and copy them there.",
            true
          );
          $error = true;
        }
      }
      foreach (self::$sectionNames as $sectionName) {
        $answerFolder = self::getanswerFolder($sectionName);
        if (is_dir($answerFolder)) {
          $answerFolder = realpath($answerFolder);
        } else {
          $str0 .= HT::error(
            "&nbsp;&rarr; Student answer folder not found:<br>
            &emsp;<code>$answerFolder</code> <br>
            Please create it so that their zip files can be unzipped.",
            true
          );
          $error = true;
        }
      }
      if ($error) {
        $str .= HT::warn($str0, true);
      } else {
        $str .= HT::info($str0, true);
      }
      $rubricFile = self::$rubricFile;
      if (!is_file($rubricFile)) {
        $str .= HT::error(
          "Rubric file not found: <br>
          &emsp;<code>$rubricFile</code><br>
          Create it before proceeding.",
          true
        );
        $error = true;
      } else {
        $rubricFile = realpath($rubricFile);
        $str .= HT::success(
          "Rubric File located:<br>
          &emsp;<code>$rubricFile</code><br>",
          true
        );
      }
      $emailFolder = self::$emailFolder;
      if (empty($emailFolder) || !is_dir($emailFolder)) {
        $str .= HT::warn(
          "Email Submission folder not found. <br> 
          &emsp;<code>$emailFolder</code><br>
          Create it and copy the emailed zip files there.",
          true
        );
      } else {
        $emailFolder = realpath(self::$emailFolder);
        $nZips = count(glob($emailFolder . "/*.zip"));
        $str .= HT::success(
          "Email Submission folder located with $nZips files:<br>
          &emsp;<code>$emailFolder</code>",
          true
        );
      }
    }
    $solutionFolder = self::$solutionFolder;
    if (!is_dir($solutionFolder)) {
      $str .= HT::error(
        "Solution folder not found: <br> 
        &emsp;<code>$solutionFolder</code><br>
        Create it and copy the reference solutions there.",
        true
      );
      $error = true;
    } else {
      $solutionFolder = realpath($solutionFolder);
      $gPattern = "$solutionFolder/{*,*/*,*/*/*}.*";
      $nFiles = count(glob($gPattern, GLOB_BRACE));
      if ($nFiles <= 0) {
        $str .= HT::warn(
          "Solution folder is empty: <br> 
        &emsp;<code>$solutionFolder</code><br>
        Copy the reference solutions there.",
          true
        );
      } else {
        $str .= HT::success(
          "Solution folder located with $nFiles files:<br> 
        &emsp;<code>$solutionFolder</code>",
          true
        );
      }
    }
    $resourceFolder = self::$resourceFolder;
    if (!is_dir($resourceFolder)) {
      $str .= HT::error(
        "Resource folder not found: <br> 
        &emsp;<code>$resourceFolder</code><br>
        Create it and copy the resource files there.",
        true
      );
      $error = true;
    } else {
      $resourceFolder = realpath($resourceFolder);
      $gPattern = "$resourceFolder/{*,*/*,*/*/*}.*";
      $nFiles = count(glob($gPattern, GLOB_BRACE));
      if ($nFiles <= 0) {
        $str .= HT::warn(
          "Resource folder is empty: <br> 
        &emsp;<code>$resourceFolder</code><br>
        Copy the resource files there.",
          true
        );
      } else {
        $str .= HT::success(
          "Resource folder located with $nFiles files:<br>
        &emsp;<code>$resourceFolder</code>",
          true
        );
      }
    }
    $gradeQuestions = self::$gradeQuestions;
    if (empty($gradeQuestions)) {
      $str .= HT::error(
        "No questions are marked gradable (<code>\$gradeQuestions</code>)! <br> 
        Edit <code>$configFile</code> to set it as an array.",
        true
      );
      $error = true;
    } else {
      $str .= HT::success(
        "Will assist in grading questions <strong>"
        . implode(", ", $gradeQuestions) . "</strong>",
        true
      );
    }
    $autoGrade = self::$autoGrade;
    if (empty($autoGrade)) {
      $str .= HT::warn(
        "No questions are marked for autograding (<code>\$autoGrade</code>)!<br> 
        Edit <code>$configFile</code> to set it if needed.",
        true
      );
    } else {
      foreach ($autoGrade as $subQuestion => $graders) {
        foreach ($graders as $grader) {
          $graderFull = "$solutionFolder/$grader";
          if (file_exists($graderFull)) {
            $str .= HT::success(
              "Autograder <code>$grader</code> will grade <code>$subQuestion</code>",
              true
            );
          } else {
            $str .= HT::error(
              "Autograder <code>$grader</code> for <code>$subQuestion</code>
            not found.<br>
            Copy it to <code>$solutionFolder</code>",
              true
            );
            $error = true;
          }
        }
      }
    }
    end:
    if ($error) {
      $configFile = self::$configFile;
      $str .= HT::error(
        "Fix the errors (red boxes) and validate again.<br>
         The comments in <a href='?do=view'><code>$configFile</code></a> may help.",
        true
      );
    }
    HT::warn($str);
    HT::foot();
  }
  public static function viewSql()
  {
    $fileName = self::$root . DIRECTORY_SEPARATOR . self::$sqlFile;
    self::mkSql();
    $ext = pathinfo($fileName, PATHINFO_EXTENSION);
    $fullText = htmlentities(file_get_contents($fileName), ENT_QUOTES);
    HT::codeHead($fileName);
    echo "<pre><code class='language-$ext'>$fullText</code></pre>";
    HT::foot();
  }
  // Danger: System calls.
  public static function rmdir($dir)
  {
    if (is_dir($dir)) {
      $files = array_diff(scandir($dir), array('.', '..'));
      foreach ($files as $file) {
        (is_dir("$dir/$file")) ? self::rmdir("$dir/$file") : unlink("$dir/$file");
      }
      return rmdir($dir);
    }
  }
  // Danger: System calls. 
  public static function mv($from, $to)
  {
    $toFolder = dirname($to);
    if (!is_dir($toFolder)) {
      mkdir($toFolder, 0777, true);
    }
    if (is_dir($to)) {
      $to .= "/" . $from;
    }
    // Make a subfolder if needed
    $error = "Error moving <code>$from</code> to <code>$to</code>:";
    if (is_dir($from)) {
      HT::error("$error Source is a folder.");
      return false;
    }
    if (!is_readable($from)) {
      HT::error("$error Source not found.");
      return false;
    }
    if (!rename($from, $to)) {
      if (copy($from, $to)) {
        if (!unlink($from)) {
          HT::error("Error moving $from to $to: Could not remove source.");
          return false;
        }
      } else {
        HT::error("Error moving $from to $to: Could not copy source.");
        return false;
      }
    }
    return true;
  }

  public static function cpr($from, $to)
  {
    if (is_dir($from)) {
      $dir_handle = opendir($from);
      while ($file = readdir($dir_handle)) {
        if ($file != "." && $file != "..") {
          if (is_dir($from . "/" . $file)) {
            if (!is_dir($to . "/" . $file)) {
              mkdir($to . "/" . $file);
            }
            self::cpr($from . "/" . $file, $to . "/" . $file);
          } else {
            copy($from . "/" . $file, $to . "/" . $file);
          }
        }
      }
      closedir($dir_handle);
    } else {
      copy($from, $to);
    }
  }

  public static function cp($from, $to, $mkDir = false)
  {
    $error = "Error copying <code>$from</code> to <code>$to</code>:";
    $error = str_replace('//', '/', $error);
    if (is_dir($from)) {
      // HT::error("$error Source is a folder.");
      // try cpr after creating the $to folder: Dicey!
      if (!is_dir($to)) {
        mkdir($to, 0777, true);
      }
      self::cpr($from, $to);
      return false;
    }
    if (!is_readable($from)) {
      HT::error("$error Source not found/readable.");
      return false;
    }
    if (is_dir($to)) {
      if (!file_exists($to)) {
        HT::error("$error Target not found/writable.");
        return false;
      }
      $to .= "/" . basename($from);
    }
    $toDir = dirname($to);
    if (!file_exists($toDir)) {
      if ($mkDir && mkdir($toDir)) {
        HT::warn("$error Target directory was not found. Created it.");
      } else {
        HT::error("$error Target directory not found/writable.");
        return false;
      }
    }
    copy($from, $to);
    return true;
  }
  public static function mkFileName($fileName)
  {
    $root = $_SERVER['DOCUMENT_ROOT'];
    $fileName0 = realpath($fileName);
    if (stripos($fileName, $root) !== 0) { // Does not start with DocRoot
      HT::error("$fileName: not found under $root.");
    }
    return substr($fileName0, strlen($root));
  }
  public static function getDbPrefix()
  {
    $dbPrefix = preg_replace("/\s+/", "", self::$examShortName);
    $dbPrefix = strtolower($dbPrefix) . "_";
    return $dbPrefix;
  }
  // Danger: System calls. To be customized for Windows.
  public static function mkSql($toString = false)
  {
    $template = self::$sqlTemplate;
    $dbPrefix = self::getDbPrefix();
    $dbName = strtolower(self::$dbName);
    $search = ["{dbName}", "{dbPrefix}"];
    $replace = [$dbName, $dbPrefix];
    $sql = file_get_contents($template);
    $sql = str_replace($search, $replace, $sql);
    $sqlFile = self::$root . DIRECTORY_SEPARATOR . self::$sqlFile;
    HT::head($toString);
    if (!file_put_contents($sqlFile, $sql)) {
      HT::error("Error creating $sqlFile while setting up the database.<br>
      Cannot safely continue without fixing this issue!");
      return "";
    }
    HT::warn("Created $sqlFile.");
    HT::foot($toString);
    return $sql;
  }
  private static function mkDropDB()
  {
    $dbPrefix = self::getDbPrefix();
    $dbName = strtolower(self::$dbName);
    $search = ["{dbName}", "{dbPrefix}"];
    $replace = [$dbName, $dbPrefix];
    $sql = 'DROP DATABASE IF EXISTS `{dbName}`;';
    $sql = str_replace($search, $replace, $sql);


    // Pattern for matching table names (SQL LIKE syntax)
    $pattern = "$dbPrefix%";

    // SQL to get the list of tables that match the wildcard pattern
    $sql0 = "SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = '$dbName' AND TABLE_NAME LIKE '$pattern'";

    $sql = "";
    if (DB::$db->query($sql0)) {
      $tables = DB::$db->fetchAll();
      foreach ($tables as $row) {
        $table = $row["TABLE_NAME"];
        $sql .= "DROP TABLE `$table`;\n";
      }
    } else {
      echo "No tables found";
    }
    // var_dump($sql); die;
    return $sql;
  }
  public static function runSql()
  {
    $template = self::$sqlTemplate;
    $sql = self::mkSql();
    if (empty($sql)) {
      HT::error("Error running $template!");
      return false;
    }
    $sqlFile = self::$root . DIRECTORY_SEPARATOR . self::$sqlFile;
    $db = new DB(dbPass: self::$dbPass);
    $status = $db->importSQL($sqlFile);
    if ($status) {
      $msg = HT::success("Executed {$status['success']} successfully.", true);
      $msg .= HT::error("{$status['errors']} statements failed.", true);
    } else {
      $msg = HT::error("Error setting up the database [CFG::runSql fails].", true);
    }
    HT::head();
    HT::warn("<h4>Result from <code>CFG::runSQL():</code></h4>" . $msg);
    HT::foot();
  }
  public static function dropDB()
  {
    HT::head();
    $sql = self::mkDropDB();
    if (empty($sql)) {
      HT::error("Error creating drop DB script!");
      return false;
    }
    $dbName = self::$dbName;
    if (!isset($_POST['confirmDropDB'])) {
      $str = "Dropping DB ($dbName)!<br><pre><code>$sql</code></pre><br>
          Really drop it? <table align='right'><tr>
          <td><form method='post' action='?do'>
          <input type='hidden' name='do' value='dropDB'>
          <input type='hidden' name='confirmDropDB' value='dummy'>
          <button type='submit' class='error'>
          <strong>Proceed to drop it</strong>
          </button></form></td>
          </tr></table>";
      HT::warn($str);
    } else {
      DB::$db->multiQuery($sql);
      $error = DB::$db->getError();
      if ($error) {
        $error = "<br>$error";
      }
      HT::error("Dropping DB ($dbName)!<br><pre><code>$sql</code></pre>$error");
      HT::warn("Please run the DB setup script again. 
      <a href='?do=runSql'>Run it!</a>");
      // self::runSql();
    }
    HT::foot();
  }
  public static function mkTableName($table)
  {
    $pf = self::getDbPrefix();
    if (strpos($table, $pf) !== 0) {
      $table = $pf . $table;
    }
    return $table;
  }
  // Utility functions: 
  // Ref: https://gist.github.com/carousel/1aacbea013d230768b3dec1a14ce5751
  public static function toSnake($input)
  {
    return strtolower(preg_replace('/(?<!^)[A-Z]/', '_$0', $input));
  }
  public static function toCamel($input)
  {
    return lcfirst(str_replace(' ', '', ucwords(str_replace('_', ' ', $input))));
  }
}