<?php
class RubricDAO
{
    private static $table;
    public function __construct()
    {
        $table = CFG::mkTableName("rubrics");
        self::$table = $table;
    }
    public function get($where = [], $verbose = false)
    {
        $rubrics = [];
        $table = self::$table;
        $rows = DB::$db->get(
            table: $table,
            where: $where
        );
        $got = count($rows);
        if ($verbose && $got <= 0) { // Not found in the DB
            HT::error("<strong>RubricDAO</strong>: No rubrics in the DB!<br>
            Run RubricDAO->process(subQuestion) for each subQuestion first.");
            return;
        } else {
            foreach ($rows as $row) {
                $rubrics[] = new Rubric(
                    $row['subquestion_name'],
                    $row['rubric_name'],
                    $row['marks'],
                    $row['rubric_text'],
                    $row['short_filename'],
                );
            }
        }
        return $rubrics;
    }
    public function getByObject($subQuestion)
    {
        $where = ['subquestion_name' => $subQuestion->getName()];
        return $this->get($where);
    }
    private function put($subQuestion, $rubrics)
    {
        $table = self::$table;
        $subQuestionName = $subQuestion->getName();
        if (count($rubrics) <= 0) { // No rubrics found. An error!
            HT::error("<strong>SubQuestion $subQuestionName</strong>: 
            No rubrics found. <br>
            Something very wrong!");
        } else {
            $columns = [
                'subquestion_name',
                'rubric_name',
                'marks',
                'rubric_text',
                'short_filename'
            ];
            $rows = [];
            foreach ($rubrics as $rubric) {
                $rubricName = $rubric->getRubricName();
                $marks = $rubric->getMarks();
                $rubricText = $rubric->getRubricText();
                $shortFileName = $rubric->getShortFileName();
                $rows[] = [
                    $subQuestionName,
                    $rubricName,
                    $marks,
                    $rubricText,
                    $shortFileName
                ];
            }
            // DB::$db->put($table, $columns, $rows);
            DB::$db->update($table, $columns, $rows);
        }
    }
    public function process($subQuestion)
    {
        $subQuestionName = $subQuestion->getName();
        // Rubrics are kept in rubric.csv
        // name, marks, rubric text
        $rubricFile = glob(CFG::$rubricFile);
        $rubricNum = 1;
        if (count($rubricFile) != 1) {
            HT::error("<strong>SubQuestion $subQuestionName</strong>: 
        Rubric File not found: " . CFG::$rubricFile);
        } else {
            $rubricFile = $rubricFile[0];
            $lines = file($rubricFile);
            $rubrics = [];
            foreach ($lines as $line) {
                $lineElements = explode(",", $line);
                $name = strtolower(trim($lineElements[0]));
                if ($name == $subQuestionName) {
                    $marks = (float) $lineElements[1];
                    $rubricText = trim($lineElements[2]);
                    $rubricText = str_replace('""', "'", $rubricText);
                    $rubricText = trim($rubricText, '"');
                    $rubricText = str_replace("'", '"', $rubricText);
                    $shortFileName = trim($lineElements[3]);
                    $shortFileName = str_replace("//", '/', $shortFileName);
                    $rubricName = $name . "-" . $rubricNum++;
                    $rubrics[] = new Rubric(
                        $name,
                        $rubricName,
                        $marks,
                        $rubricText,
                        $shortFileName
                    );
                }
            }
            $this->put($subQuestion, $rubrics);
        }
    }
}