<?php
/**
 * Summary of StudentDAO
 */
class StudentDAO
{
    private static $table;
    private static $collate = 'student_email';
    private static $students = [];
    private static $iStudent = [];
    private static $nStudents = 0;
    public function __construct()
    {
        $table = CFG::mkTableName("students");
        self::$table = $table;
        if (empty(self::$students)) {
            $students = $this->get();
            self::$students = $students;
            self::$nStudents = count($students);
            $i = 0;
            foreach ($students as $student) {
                $i++;
                self::$iStudent[$student->getEmail()] = $i;
            }
            if (self::$nStudents != $i) {
                xdebug_print_function_stack();
                die("Error in student arrays");
            }
            ksort(self::$iStudent);
        }
    }
    /**
     * Summary of get
     * @param [] $where Assoc array for SQL WHERE clause
     * @return []|Student
     */
    public function get($where = [], $verbose = false)
    {
        $table = self::$table;
        $rows = DB::$db->get(
            table: $table,
            where: $where
        );
        $got = count($rows);
        $students = [];
        if ($verbose && $got <= 0) { // Not found in the DB
            HT::error("<strong>StudentDAO</strong>: No students in the DB!<br>
            Run StudentDAO->process(section) for each section first.");
        } else {
            foreach ($rows as $row) {
                $student =new Student(
                    $row['student_email'],
                    $row['student_name'],
                    $row['section']
                );
                $student->setZipFile($row['zip_filename']);
                $student->setStatus($row['status']);
                $student->setComment($row['comment']);
                $students[] = $student;
            }
        }
        return $students;
    }
    /**
     * Given an asnwer, get the student. Should be only one.
     * @param string $studentEmail
     * @return []
     */
    public function getByEmail($studentEmail)
    {
        $student = $this->get(['student_email' => $studentEmail]);
        if (count($student) > 1) {
            HT::error("Too many students with email id $studentEmail!");
            $student = $student[0];
        }
        return $student;
    }
    public function getIStudent($studentEmail)
    {
        return self::$iStudent[$studentEmail];
    }
    /**
     * Given an asnwer, get the student. Should be only one.
     * @param Answer $answer
     * @return []
     */
    public function getByObject($answer)
    {
        return $this->getByEmail($answer->getStudentEmail());
    }
    public function getOthers($student)
    {
        $others = $this->get();
        $needPrevious = true;
        $needNext = false;
        $prev = $prevUngraded = $nextUngraded = $next = "";
        foreach ($others as $other) {
            $status = $other->getStatus();
            if ($status == 'Absent') {
                continue;
            }
            if ($other->getEmail() == $student->getEmail()) {
                $needPrevious = false;
                $needNext = true;
                continue;
            }
            if ($needPrevious) {
                $prev = $other;
                if ($status == "New") {
                    $prevUngraded = $other;
                }
            }
            if ($needNext) {
                if ($next == "") {
                    $next = $other;
                }
                if ($status == "New") {
                    $nextUngraded = $other;
                    break;
                }
            }
        }
        return [$prev, $prevUngraded, $nextUngraded, $next];
    }
    /**
     * Save all the students in the given section in the DB
     * @param mixed $section
     * @return void
     */
    public function put($section, $students)
    {
        $table = self::$table;
        $sectionName = $section->getName();
        if (count($students) <= 0) { // No students found. An error!
            HT::error("<strong>Section $sectionName</strong>: 
            No students found. <br>
            Something very wrong!");
        } else {
            $columns = ['student_email', 'student_name', 'section', 'status', 'zip_filename', 'comment'];
            $rows = [];
            foreach ($students as $student) {
                $studentEmail = $student->getEmail();
                $studentName = $student->getName();
                $status = $student->getStatus();
                $zipFileName = $student->getZipFile();
                $comment = $student->getComment();
                $rows[] = [$studentEmail, $studentName, $sectionName, $status, $zipFileName, $comment];
            }
            // DB::$db->put($table, $columns, $rows);
            DB::$db->update($table, $columns, $rows);
        }
    }
    public function update($student)
    {
        $table = self::$table;
        $columns = ['student_email', 'student_name', 'section', 'status', 'zip_filename', 'comment'];
        $studentEmail = $student->getEmail();
        $studentName = $student->getName();
        $section = $student->getSection();
        $status = $student->getStatus();
        $zipFileName = $student->getZipFile();
        $comment = $student->getComment();
        $rows = [[$studentEmail, $studentName, $section, $status, $zipFileName, $comment]];
        // DB::$db->put($table, $columns, $rows);
        DB::$db->update($table, $columns, $rows);
    }
    public function process($section)
    {
        $sectionName = $section->getName();
        $csv = CFG::getCsvFileName($sectionName);
        if (!is_file($csv)) {
            HT::error("Error loading $csv");
            return;
        }
        $file = fopen($csv, 'r');
        $line = fgetcsv($file); // Skip the header
        $students = [];
        while (($line = fgetcsv($file)) !== FALSE) {
            //$line is an array of the csv elements
            $students[] = new Student($line[0], $line[1], $sectionName);
        }
        fclose($file);
        $this->put($section, $students);
    }

    /**
     * @return mixed
     */
    public static function getNStudents()
    {
        return self::$nStudents;
    }

	/**
	 * @return mixed
	 */
	public static function getStudents() {
		return self::$students;
	}
}