<?php
/**
 * Summary of DB
 */
class DB
{
  // https://codeshack.io/super-fast-php-mysql-database-class/
  // Modified: 
  //   (1) Automate DB setup in the constructor using try-catch
  //   (2) Add some utility methods
  protected $connection;
  protected $query;
  protected $show_errors = TRUE;
  protected $query_closed = TRUE;
  public $query_count = 0;
  public static $db;
  private $error = "";
  private static $orderBy = ["section", "student_email"];
  public function __construct(
    $dbHost = 'localhost',
    $dbUser = 'root',
    $dbPass = '',
    $dbName = '',
    $charset = 'utf8'
  ) {
    try {
      mysqli_report(MYSQLI_REPORT_ERROR | MYSQLI_REPORT_STRICT);
      $configFile = realpath("config.php");
      $setupMsg = "Edit the config file <code>$configFile</code>
      and go to <strong>Setup and Configuration</strong> and set it up.";
      $this->connection = new mysqli($dbHost, $dbUser, $dbPass, $dbName);
      if ($this->connection->connect_error) { // For WAMP
        $error = $this->connection->connect_error;
        $this->error = "WAMP: Database connection error: $error. 
        $setupMsg";
      } else {
        $this->connection->set_charset($charset);
        self::$db = $this;
      }
    } catch (Exception $e) { // for MAMP
      $error = $e->getMessage();
      $configFile = realpath("config.php");
      $this->error = "MAMP: Database connection error: $error. 
      $setupMsg";
    }
  }
  // Manoj's utility methods
  public function sanitize($str)
  {
    return $this->connection->real_escape_string($str);
  }
  public function getFileContent($file)
  {
    return $this->sanitize(file_get_contents($file));
  }
  /**
   * Summary of get
   * @param string $table Full table name (with prefix).
   * @param string|string[] $what Array of columns to get. Empty => *.
   * @param [] $where Associative array: column => value. Empty => no WHERE clause.
   * @return [] Collated output
   */
  public function get($table, $what = '', $where = [])
  {
    if (is_array($what)) {
      if (in_array("*", $what)) {
        $what = [];
      }
      $columns = $what;
    } else {
      if (empty($what) || $what == "*") {
        $columns = [];
      } else {
        $columns = [$what];
      }
    }
    if (empty($columns)) {
      $what = "*";
    } else {
      $what = "`" . implode("`, `", $columns) . "`";
    }
    $keys = $where;
    if (empty($keys)) {
      $where = "";
    } else {
      $where = [];
      foreach ($keys as $key => $val) {
        $where[] = "`$key` = '$val'";
      }
      $where = "WHERE " . implode(" AND ", $where);
    }
    $sql = "SELECT $what FROM $table $where";
    $orderBy = $this->getOrderBy($table, self::$orderBy);
    $sql .= "$orderBy";
    $rows = $this->query($sql)->fetchAll();
    return $rows;
  }
  private static function mkSql($table, $columns, $rows, $insert = "REPLACE")
  {
    $values = [];
    foreach ($rows as $key => $row) {
      if (is_array($row)) { // Multiple rows
        $values[$key] = "('" . implode("', '", $row) . "')";
      } else { // Single row
        $values = "('" . implode("', '", $rows) . "')";
      }
    }
    if (is_array($values)) { // Multiple rows
      $values = implode(",\n ", $values);
    }
    $sql = "$insert INTO $table 
        ( `" . implode("`, `", $columns) . "`) 
        VALUES $values";
    return $sql;
  }
  public function put($table, $columns, $rows)
  {
    $sql = self::mkSql($table, $columns, $rows, "REPLACE");
    $this->query($sql);
  }
  public function update($table, $columns, $rows)
  {
    $sql = self::mkSql($table, $columns, $rows, "INSERT");
    $update = "";
    foreach ($columns as $column) {
      $update .= "`$column`=VALUES(`$column`), ";
    }
    $update = trim($update, ", ");
    $sql .= " ON DUPLICATE KEY UPDATE $update";
    $this->query($sql);
  }
  public function getOrderBy($table, $columns)
  {
    if (!is_array($columns)) {
      $columns = [$columns];
    }
    foreach ($columns as $key => $column) {
      if (!$this->colExists($table, $column)) {
        unset($columns[$key]);
      }
    }
    if (count($columns) > 0) {
      $orderBy = "ORDER BY `" . implode("`, `", $columns) . "`";
    } else {
      $orderBy = "";
    }
    return $orderBy;
  }
  public function colExists($table, $column)
  {
    $sql = "SELECT * 
      FROM `information_schema`.COLUMNS 
      WHERE TABLE_NAME = '$table' 
      AND COLUMN_NAME = '$column'";
    return !empty($this->query($sql)->fetchAll());
  }
  public function tablesExist($dbPrefix)
  {
    $sql = "SHOW TABLES LIKE '$dbPrefix%'";
    $results = $this->query($sql);
    $results = $results->fetchAll();
    return !empty($results);
  }
  public function getEnum($table, $field)
  {
    $sql = "SHOW COLUMNS FROM {$table} WHERE Field = '{$field}'";
    $result = self::$db->query($sql);
    $row = $result->fetchAll();
    $type = $row[0]['Type'];
    preg_match('/enum\((.*)\)$/', $type, $matches);
    $vals = explode(',', $matches[1]);
    return $vals;
  }
  //// End of Manoj's utility methods
  public function query($query)
  {
    if (!$this->query_closed) {
      $this->query->close();
    }
    if ($this->query = $this->connection->prepare($query)) {
      if (func_num_args() > 1) {
        $x = func_get_args();
        $args = array_slice($x, 1);
        $types = '';
        $args_ref = array();
        foreach ($args as $k => &$arg) {
          if (is_array($args[$k])) {
            foreach ($args[$k] as $j => &$a) {
              $types .= $this->_gettype($args[$k][$j]);
              $args_ref[] = &$a;
            }
          } else {
            $types .= $this->_gettype($args[$k]);
            $args_ref[] = &$arg;
          }
        }
        array_unshift($args_ref, $types);
        call_user_func_array(array($this->query, 'bind_param'), $args_ref);
      }
      $this->query->execute();
      if ($this->query->errno) {
        $this->error('Unable to process MySQL query (check your params) - ' . $this->query->error);
      }
      $this->query_closed = FALSE;
      $this->query_count++;
    } else {
      $this->error('Unable to prepare MySQL statement (check your syntax) - ' . $this->connection->error);
    }
    return $this;
  }


  public function fetchAll($callback = null)
  {
    $params = array();
    $row = array();
    $meta = $this->query->result_metadata();
    while ($field = $meta->fetch_field()) {
      $params[] = &$row[$field->name];
    }
    call_user_func_array(array($this->query, 'bind_result'), $params);
    $result = array();
    while ($this->query->fetch()) {
      $r = array();
      foreach ($row as $key => $val) {
        $r[$key] = $val;
      }
      if ($callback != null && is_callable($callback)) {
        $value = call_user_func($callback, $r);
        if ($value == 'break')
          break;
      } else {
        $result[] = $r;
      }
    }
    $this->query->close();
    $this->query_closed = TRUE;
    return $result;
  }

  public function fetchArray()
  {
    $params = array();
    $row = array();
    $meta = $this->query->result_metadata();
    while ($field = $meta->fetch_field()) {
      $params[] = &$row[$field->name];
    }
    call_user_func_array(array($this->query, 'bind_result'), $params);
    $result = array();
    while ($this->query->fetch()) {
      foreach ($row as $key => $val) {
        $result[$key] = $val;
      }
    }
    $this->query->close();
    $this->query_closed = TRUE;
    return $result;
  }

  public function close()
  {
    return $this->connection->close();
  }

  public function numRows()
  {
    $this->query->store_result();
    return $this->query->num_rows;
  }

  public function affectedRows()
  {
    return $this->query->affected_rows;
  }

  public function lastInsertID()
  {
    return $this->connection->insert_id;
  }

  public function error($error)
  {
    if ($this->show_errors) {
      exit($error);
    }
  }

  private function _gettype($var)
  {
    if (is_string($var))
      return 's';
    if (is_float($var))
      return 'd';
    if (is_int($var))
      return 'i';
    return 'b';
  }

  /**
   * @return mixed
   */
  public function getError()
  {
    return $this->error;
  }

  // To import an sql: From my own DbHelper.php, which took it from phpBB
  public static function remove_comments(&$output)
  {
    $lines = explode("\n", $output);
    $output = "";

    $linecount = count($lines);

    $in_comment = false;
    for ($i = 0; $i < $linecount; $i++) {
      if (preg_match("/^\/\*/", preg_quote($lines[$i]))) {
        $in_comment = true;
      }

      if (!$in_comment) {
        $output .= $lines[$i] . "\n";
      }

      if (preg_match("/\*\/$/", preg_quote($lines[$i]))) {
        $in_comment = false;
      }
    }

    unset($lines);
    return $output;
  }

  // From phpBB. Modified.
  // remove_remarks will strip the sql comment lines out of an uploaded sql file
  public static function remove_remarks($sql)
  {
    $lines = explode("\n", $sql);
    $linecount = count($lines);
    $output = "";

    for ($i = 0; $i < $linecount; $i++) {
      if (($i != ($linecount - 1)) || (strlen($lines[$i]) > 0)) {
        if (isset($lines[$i][0]) && $lines[$i][0] != "#") {
          $output .= $lines[$i] . "\n";
        } else {
          $output .= "\n";
        }
      }
    }

    return $output;
  }

  public static function remove_inline($sql)
  {
    $regex = array('/\/\*.*\*\//U');
    $sql = preg_replace($regex, "", $sql);
    $sql = trim($sql);
    return $sql;
  }
  public static function split_sql_file($sql, $delimiter = ";")
  {
    $sql = trim($sql);
    $tokens = explode($delimiter, $sql);
    $output = array();
    $matches = array();

    // this is faster than calling count($oktens) every time thru the loop.
    $token_count = count($tokens);
    for ($i = 0; $i < $token_count; $i++) {
      $tokens[$i] = self::remove_inline($tokens[$i]);
      // Don't wanna add an empty string as the last thing in the array.
      if (($i != ($token_count - 1)) || (strlen($tokens[$i] > 0))) {
        // This is the total number of single quotes in the token.
        $total_quotes = preg_match_all("/'/", $tokens[$i], $matches);
        // Counts single quotes that are preceded by an odd number of backslashes,
        // which means they're escaped quotes.
        $escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $tokens[$i], $matches);

        $unescaped_quotes = $total_quotes - $escaped_quotes;

        // If the number of unescaped quotes is even, then the delimiter did NOT occur inside a string literal.
        if (($unescaped_quotes % 2) == 0) {
          // It's a complete sql statement.
          $output[] = $tokens[$i];
        } else {
          // incomplete sql statement. keep adding tokens until we have a complete one.
          // $temp will hold what we have so far.
          $temp = $tokens[$i] . $delimiter;

          // Do we have a complete statement yet?
          $complete_stmt = false;

          for ($j = $i + 1; (!$complete_stmt && ($j < $token_count)); $j++) {
            // This is the total number of single quotes in the token.
            $total_quotes = preg_match_all("/'/", $tokens[$j], $matches);
            // Counts single quotes that are preceded by an odd number of backslashes,
            // which means they're escaped quotes.
            $escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $tokens[$j], $matches);

            $unescaped_quotes = $total_quotes - $escaped_quotes;

            if (($unescaped_quotes % 2) == 1) {
              // odd number of unescaped quotes. In combination with the previous incomplete
              // statement(s), we now have a complete statement. (2 odds always make an even)
              $output[] = $temp . $tokens[$j];

              // exit the loop.
              $complete_stmt = true;
              // make sure the outer loop continues at the right point.
              $i = $j;
            } else {
              // even number of unescaped quotes. We still don't have a complete statement.
              // (1 odd and 1 even always make an odd)
              $temp .= $tokens[$j] . $delimiter;
            }
          } // for..
        } // else
      }
    }
    foreach ($output as $k => $o) {
      $o = trim($o);
      if (empty($o)) {
        unset($output[$k]);
      } else {
        $output[$k] = $o;
      }
    }
    return $output;
  }

  public function multiQuery($mSql)
  {
    $mSql = self::remove_comments($mSql);
    $mSql = self::remove_remarks($mSql);
    $lines = self::split_sql_file($mSql);
    $errors = $success = 0;
    $message = "";
    foreach ($lines as $sql) {
      try {
        $this->connection->query($sql, MYSQLI_STORE_RESULT);
        ++$success;
      } catch (Exception $e) {
        ++$errors;
        $message .= "\n$errors: " . $e->getMessage();
      }
    }
    $message = trim($message);
    return compact('success', 'errors', 'message');
  }

  public function importSQL($sqlFile, $gzip = false)
  {
    if (!is_readable($sqlFile)) {
      return false;
    }
    $sql = file_get_contents($sqlFile);
    if ($gzip) {
      $sql = gzdecode($sql);
    }
    return $this->multiQuery($sql);
  }
}