<?php

/**
 * Summary of Grader 
 */
class Grader extends HT
{
  protected static $class = __CLASS__;
  protected static $actions = [
    'load' => 'Load Student List',
    'view' => 'View Marks',
    'export' => 'Export Grades (CSV)',
    'autoGrade' => 'Auto-Grade',
  ];
  protected static $actionProperties = [
    'load' => [
      'target' => 'left',
      'clear' => 'yes'
    ],
    'view' => [
      'target' => 'right'
    ],
    'export' => [
      'target' => 'right'
    ],
    'autoGrade' => [
      'target' => 'left',
      'class' => 'error',
      'clear' => 'yes'
    ]
  ];
  protected static $actionDesc = [
    'load' => 'Load Student List so that you can grade each one. The list is
    color-coded to indicate the students who were absent for the test. 
    Once loaded, a student can be graded by clicking on their 
    <button>Grade</button> or <button>Continue</button> button. 
    <span style="color:red">Note that the answer files viewed and the output shown
    are from the current/edited versions (from the disk).</span>',
    'export' => 'After the grading is complete, click on this button to 
    generate CSV files that can be imported to eLearn. The file names will
    be <code>Export-{gradebook}.csv</code> where <code>{gradebook}</code> 
    is the name of the grade book exported from eLearn. The files will be 
    in the same  folder as the grade book exports.',
    'view' => 'View the marks that will be exported when the next button 
    is clicked.',
    'autoGrade' => 'Run the autograder (typically for Q3, but as specified in 
    <code>config.php</code>) for all students. It will update the database
    with the marks computed. The autograder file should output the total marks 
    in some recognizable form, as specified in  <code>config.php &rarr; 
    $autoGradeOptions</code>. The options should also specify the subquestion
    name (usually q3) and the rubric name (usually q3-1), so that the marks
    can be inserted into the database. For those students whose file crashes,
    Grader will put zero in the database, but will give you buttons to view
    and manually grade q3.'
  ];
  protected static $intro = "<h4>Grade Students</h4>
    <p>The most time-consuming process, grading the students, is triggered here. 
    Click on the load button to get a list of students, and click on the
    buttons in the list to grade/regrade etc. Grader can export the marks that 
    can be directly imported to eLearn.</p>
  ";
  protected static $display = [
    "class" => "warn",
    "title" => "Grade Students"
  ];
  private static function mkPopup($file)
  {
    return "onclick=
      \"var h = screen.height * 0.8;
      var t = screen.height * 0.05;
      var w = screen.width * 0.4;
      var l = screen.width * 0.05;
      window.open('', '$file', 'width='+ w +',height=' + h + ',left='+ l +',top='+ t +',resizable=yes,scrollbars=yes,toolbar=yes,menubar=no,location=no,directories=no,status=yes'); 
        document.getElementById('$file').submit();
        return false;\"";
  }
  private static function mkCommentPopup()
  {
    $file = "editComment";
    return "onclick=
      \"var h = screen.height * 0.3;
      var t = screen.height * 0.05;
      var w = screen.width * 0.25;
      var l = screen.width * 0.65;
      window.open('', '$file', 'width='+ w +',height=' + h + ',left='+ l +',top='+ t +',resizable=yes,scrollbars=yes,toolbar=yes,menubar=no,location=no,directories=no,status=yes'); 
        document.getElementById('$file').submit();
        return false;\"";
  }
  private static function showAnswers($student)
  {
    $wrap = 5;
    $srcDoc = HT::head(true);
    $answers = self::$answerDao->getByObject($student);
    $studentName = $student->getName();
    $tabAns = "<table align='center' title='Grader::showAnswers()'><tr>
      <td colspan='100%' style='text-align:center;font-size:120%;'>
      Answers: <strong title='Grader::showAnswers()'>$studentName</strong>
      </td></tr><tr>";
    $loadAnswers = "";
    $iAns = 0;
    if (count($answers) > 0) { // Student attempted
      $firstAnswer = "id='firstAnswer'";
      foreach ($answers as $answer) {
        $td = $answer->render($firstAnswer);
        if ($td) {
          $tabAns .= $td;
          $firstAnswer = "";
          $iAns++;
          if ($iAns % $wrap == 0) {
            $tabAns .= "</tr><tr>";
          }
        }
      }
      $loadAnswers = "<script type='text/javascript'>
          window.onload=function()
          {document.getElementById('firstAnswer').click();} 
          </script>";
    } else { // Absent?
      $tabAns .= "<td>Not Attempted</td>";
    }
    $colSpan = $wrap - $iAns % $wrap;
    if ($colSpan) {
      $colSpan = "colspan='$colSpan'";
    } else {
      $colSpan = "";
    }
    $closeTable = "<th $colSpan><button onclick='history.back()' class='success'>
    <strong>Go Back</strong></button></th></tr></table>";
    $tabAns .= $closeTable;
    $srcDoc .= HT::info($tabAns, true);
    $srcDoc .= $loadAnswers;
    $srcDoc .= HT::foot(true);
    return $srcDoc;
  }
  protected static function putMarks()
  {
    $full = isset($_POST['putFullMarks']);
    self::init();
    $row = $_POST;
    $subQuestionName = $row['subquestion_name'];
    $subQuestion = self::$subQuestionDao->get(
      ['subquestion_name' => $subQuestionName]
    )[0];
    $rubrics = self::$rubricDao->getByObject($subQuestion);
    $studentEmail = $row['student_email'];
    $student = self::$studentDao->getByEmail($studentEmail)[0];
    $rubSrc = "<table class='grader' width='80%' align='center'>
      <tr><th>Rubric</th><th>Award</th><th>Ref</th></tr>";
    $totAwarded = $totQuestion = 0.0;
    foreach ($rubrics as $rubric) {
      $rubricName = $rubric->getRubricName();
      if (isset($row[$rubricName])) {
        $marks = self::$marksDao->getByObject($student, $subQuestion, $rubric);
        if (empty($marks)) {
          $marks = new Marks($studentEmail, $subQuestionName, $rubricName, 0);
        } else {
          $marks = $marks[0];
        }
        $rubricName = $rubric->getRubricName();
        if ($full) {
          $awarded = (float) $row["full-$rubricName"];
        } else {
          $awarded = (float) $row[$rubricName];
        }
        $marks->setMarks($awarded);
        self::$marksDao->update($marks);
        $refMarks = $rubric->getMarks();
        $rubricText = $rubric->getRubricText();
        $max = max([$refMarks, 0]);
        if ($awarded == $max) {
          $class = "success";
        } else if ($awarded > 0) {
          $class = "warn";
        } else {
          $class = "error";
        }
        $totQuestion += $max;
        $totAwarded += $awarded;
        $rubricText = htmlentities($rubricText, ENT_QUOTES);
        $rubSrc .= "<tr class='$class'>
        <td style='text-align:left;'> $rubricName: <var>$rubricText</var></td>
        <td>$awarded</td><td> $awarded </td> </tr>";
      }
    }
    $rubSrc .= "<tr><th>Total</th><th>$totAwarded</th>
    <th>$totQuestion</th></tr></table>";
    $reload = "<script type='text/javascript'>var done='no';
      window.onload=function()
      {window.parent.parent.left.left1.document.getElementById('reload').submit(); 
      /* alert('Marks Updated: [$subQuestionName = $totAwarded/$totQuestion]');*/} 
      </script>";
    echo $reload;
    // HT::success($rubSrc);
    // HT::foot();
  }
  private static function getDefaultComment($answer)
  {
    self::init();
    $subQuestionName = $answer->getSubQuestionName();
    $shortFileName = $answer->getShortFileName();
    $rubrics = self::$rubricDao->get([
      'subquestion_name' => $subQuestionName,
      'short_filename' => $shortFileName
    ]);
    $comment = "";
    foreach ($rubrics as $rubric) {
      $comment .= $rubric->getRubricName() . " [" .
        $rubric->getRubricText() . "]: \n\n";
    }
    return $comment;
  }

  public static function restoreFile()
  {
    self::init();
    $fileName = $_POST['filename'];
    $answer = self::$answerDao->getByFileName($fileName);
    $nAnswer = count($answer);
    if ($nAnswer == 0) {
      HT::error("Sorry, could not find the answer in the DB!<br>
              File: $fileName. This should never happen.");
      return;
    } elseif ($nAnswer > 1) {
      HT::error("Too many answers for $fileName!<br>
              This should never happen.");
      return;
    }
    $answer = $answer[0];
    $fileText = file_get_contents($fileName);
    $fileDB = $answer->getFullText();
    $str = "";
    if ($fileText == $fileDB) {
      $str .= HT::warn("Text in the file is identical to the one in the DB. Nothing to restore!", true);
    } else {
      $pathInfo = pathinfo($fileName);
      $to = "{$pathInfo['dirname']}/{$pathInfo['filename']}-edited.{$pathInfo['extension']}";
      if (file_exists($to)) {
        $str .= HT::error("Target file $to exists. Please keep it safe before trying to overwrite it!", true);
      } else {
        if (CFG::cp($fileName, $to)) {
          $str .= HT::success("Copied $fileName to $to successfully!", true);
        }
      }
      if (file_put_contents($fileName, $fileDB)) {
        $str .= HT::warn("Restored student update from DB to $fileName. (Overwrote it!)", true);
      }
    }
    HT::head();
    echo $str;
    HT::foot();
    return;
  }
  public static function editComment()
  {
    self::init();
    $fileName = $_POST['filename'];
    $answer = self::$answerDao->getByFileName($fileName);
    $nAnswer = count($answer);
    if ($nAnswer == 0) {
      HT::error("Sorry, could not find the answer in the DB!<br>
              File: $fileName. This should never happen.");
      return;
    } elseif ($nAnswer > 1) {
      HT::error("Too many answers for $fileName!<br>
              This should never happen.");
      return;
    }
    $answer = $answer[0];
    $studentEmail = $answer->getStudentEmail();
    $student = self::$studentDao->getByEmail($studentEmail)[0];
    $studentName = $student->getName();
    $comment = $answer->getComment();
    if (empty($comment)) {
      $comment = self::getDefaultComment($answer);
    }
    $msg = $student->getMessage();
    $str = HT::info("<form method='post'>
      <table width='90%' align='center'>
      <tr><td width='20%'>Name: </td>
      <td><input type='text' value='$studentName $msg' 
      style='width:90%' disabled></td></tr>
      <tr><td>File: </td>
      <td><input type='text' value='$fileName' style='width:90%' disabled></td>
      </tr><tr>
      <td valign='top'>Comment:</td>
      <td>
      <textarea style='width:90%;height:70vh;' name='comment'>$comment</textarea>
      </td>
      </tr>
      <tr>
      <th colspan='100%'><input type='submit' value='Update Comment' name='postComment' 
      align='center'></th>
      </tr>
      </table>
      <input type='hidden' name='do' value='postComment'>
      <input type='hidden' name='student_email' value='$studentEmail'>
      <input type='hidden' name='filename' value='$fileName'>
      </form>", true);
    HT::head();
    echo $str;
    HT::foot();
    return;
  }
  public static function postComment()
  {
    self::init();
    $fileName = $_POST['filename'];
    $answer = self::$answerDao->getByFileName($fileName);
    $nAnswer = count($answer);
    if ($nAnswer == 0) {
      HT::error("Sorry, could not find the answer in the DB!<br>
              File: $fileName. This should never happen.");
      return;
    } elseif ($nAnswer > 1) {
      HT::error("Too many answers for $fileName!<br>
              This should never happen.");
      return;
    }
    $answer = $answer[0];
    $studentEmail = $answer->getStudentEmail();
    $student = self::$studentDao->getByEmail($studentEmail)[0];
    $studentName = $student->getName();
    $comment = $_POST['comment'];
    $str = HT::success("
      <table width='90%' align='center'>
      <tr><td width='20%'>Name: </td><td><input type='text' value='$studentName'
      style='width:90%' disabled></td></tr>
      <tr><td>File: </td><td><input type='text' value='$fileName'
      style='width:90%' disabled></td></tr>
      <tr><td valign='top'>Comment:</td><td>
      <textarea style='width:90%;height:70vh;' disabled>$comment</textarea>
      </td></tr>
      <tr><th colspan='100%'>Comment Posted to the DB</th></tr>
      </table>");
    $comment = DB::$db->sanitize($_POST['comment']);
    $answer->setComment($comment);
    self::$answerDao->update($answer);
    HT::head();
    echo $str;
    HT::foot();
    return;
  }
  protected static function viewFile()
  {
    $fileName = $_POST['filename'];
    if (isset($_POST['showDB'])) {
      $fulltext = $_POST['fulltext'];
      $output = "<form method='post' id='restoreFile'>
      <input type='hidden' name='do' value='restoreFile'>
      <input type='hidden' name='filename' value='$fileName'>
      <input type='submit' name='comment' value='Restore File'>
      </form>";
    } else {
      $fulltext = file_get_contents($fileName);
      $fileName0 = CFG::mkFileName($fileName);
      $output = "<a href='$fileName0'>
    <input type='submit' name='run' value='View Output'></a>";
    }
    $fulltext = htmlentities($fulltext);
    $ext = pathinfo($fileName, PATHINFO_EXTENSION);
    $commentable = isset($_POST['commentable']);
    HT::codeHead($fileName);
    $comment = "";
    if ($commentable) {
      $popup = $_POST['popup'];
      $student_email = $_POST['student_email'];
      $subquestion_name = $_POST['subquestion_name'];
      $comment = "<form method='post' id='editComment' target='editComment'>
      <input type='hidden' name='do' value='editComment'>
      <input type='hidden' name='filename' value='$fileName'>
      <input type='hidden' name='popup' value='$popup'>
      <input type='hidden' name='student_email' value='$student_email'>
      <input type='hidden' name='subquestion_name' value='$subquestion_name'>
      <input type='submit' name='comment' value='Edit Comment' $popup>
      </form>";
    }
    $realPath = "<div 
    title='Grader::viewFile()'
    style='color: #373;
    background: #ecffd6;
    border: 1px solid #617c42;
    padding:4px;
    font-weight:bold;
    text-align:left;'><code>" .
      CFG::mkFileName($fileName) . "</code></div>";
    echo "<table padding='20' style='position:absolute;top:60px;right:15px;'>
    <tr><td>$comment</td><td>$output</td></tr>
    </table>";
    echo "$realPath<pre><code class='language-$ext'>$fulltext</code></pre>";
    HT::foot();
  }
  // Dispatch handlers
  protected static function export()
  {
    self::init();
    $exam = self::$exam;
    $exam->export();
  }
  protected static function view()
  {
    self::init();
    $exam = self::$exam;
    $exam->csv();
  }
  public static function load()
  {
    self::init();
    HT::head();
    self::$exam->grade();
    HT::foot();
  }
  public static function gradeOne()
  {
    $email = $_POST['email'];
    self::init();
    $student = self::$studentDao->getByEmail($email)[0];
    $left1 = htmlentities($student->render(), ENT_QUOTES);
    $questions = self::$exam->getQuestions();
    $left2 = "";
    foreach ($questions as $question) {
      $left2 .= htmlentities($question->render($student), ENT_QUOTES);
    }
    HT::head();
    echo "<iframe name='left1' srcdoc='$left1' 
    style='border:none;width:98vw;height:100px;min-height:100px;' ></iframe>";
    echo "<iframe name='left2' srcdoc='$left2' 
    style='border:none;width:98vw;height:90vh;' ></iframe>";
    HT::foot();
  }
  public static function quickSummary()
  {
    $email = $_POST['email'];
    self::init();
    $student = self::$studentDao->getByEmail($email)[0];
    $msg = htmlentities(
      HT::head(true) .
        HT::info($student->getMessage(), true) .
        HT::warn(html_entity_decode($student->getComment()), true) .
        HT::foot(true),
      ENT_QUOTES
    );
    echo "<iframe name='left2' srcdoc='$msg'
    style='border:none;width:98vw;height:90vh;'  ></iframe>";
  }
  public static function loadAnswers()
  {
    $email = $_POST['email'];
    self::init();
    $student = self::$studentDao->getByEmail($email)[0];
    // $right1 = htmlentities(self::showSummary($student));
    $right1 = htmlentities(self::showAnswers($student), ENT_QUOTES);
    HT::head();
    echo "<iframe name='right1' srcdoc='$right1' 
    style='border:none;width:98vw;height:150px;min-height:150px;' ></iframe>";
    echo "<iframe name='right2' srcdoc='' 
    style='border:none;width:98vw;height:85vh;' ></iframe>";
    HT::foot();
  }
  public static function autoGrade()
  {
    self::init();
    HT::head();
    $exam = self::$exam;
    foreach ($exam->getSections() as $section) {
      foreach ($section->getStudents() as $student) {
        $student->autoGrade();
      }
    }
    HT::foot();
  }
}
