<?php

/**
 * Summary of HT
 */
class HT
{
  protected static $graderPath;
  private static $ico = [
    "plain" => "",
    "info" => "&#9728;",
    "success" => "&#10004;",
    "warn" => "&#9888;",
    "error" => "&#9747;"
  ];
  protected static $class = __CLASS__;
  protected static $actions = [
    'setup' => 'Setup and Validate',
    'process' => 'Process Submissions, Rubrics etc.',
    'grade' => 'Grade Students',
    'stats' => 'View Grade Statistics',
    'report' => 'Investigate'
  ];
  protected static $actionProperties = [];
  protected static $actionDesc = [
    'setup' => 'Set up the database, inspect and validate the configuration file
    <code>config.php</code>. If you see a database setup warning above, click
    on this button to verify your configuration, validate it, examine the SQL 
    file and set up your database.',
    'process' => 'Import <code>rubrics.csv</code> to create rubrics, import
    the grade book downloads from eLearn to create questions, subquestions and
    students, and import student submissions to create answers and prepare
    for grading. You can also use Grader to verify that students have submitted
    valid zip files',
    'grade' => 'The next phase (the most time-consuming one) is to grade the 
    students. Clicking on this button will make the process as painless 
    as possible.',
    'stats' => 'View Grade Statistics',
    'report' => 'Once the grading process is completed, click on this button
    to export the grades to a CSV file (which can be imported to eLearn 
    with minimal modifications). It also has some heuristics to detect 
    potential cheating efforts and inconsistencies (such as missing or 
    wrong emails in student files as well as similarities among them).'
  ];
  protected static $intro = "<h4>Grader V2</h4>
  <p>Grader is a grading assistant for IS113. It follows a menu system using 
  buttons. Each button above brings up another page with a bunch of action 
  buttons. On each page (including this one), you click the buttons roughly 
  from left to right to complete your grading process. And each page has its 
  own help text, like this one.</p>
  <p>It can do the following:</p>
  <ul>
      <li>
          Read the grade book files (downloaded from eLearn as CSV) and 
          autogenerate questions, subquestions and their marks.
      </li>
      <li>
          Read the same grade books to generate a list of students to be graded.
      </li>
      <li>
          Unpack and validate the answer zip files the students (as downloaded 
          from eLearn assignment folders), identifying students who are absent
          or had issues zipping/uploading.
      </li>
      <li>
          Generate rubrics based on <code>rubrics.csv</code> 
          (to be created by hand).
      </li>
      <li>
          Let you view and test the students' answers and their outputs and 
          compare against reference solutions.
      </li>
      <li>
          Autograde specified questions using the test cases provided.
      </li>
      <li>
          Store the marks entered against each rubric in the database.
      </li>
      <li>
          Create CSV export files to be uploaded back to eLearn.
      </li>
      <li>
          Compare student answers and identify possible cheating attempts.
      </li>
      <li>
          And more that I may have forgotten.
      </li>
  </ul>
  <p>
      <strong>
          Preparation
      </strong>
  </p>
  <ul>
      <li>
        Create a folder for your lab test. Example: <code>./LT2</code> within 
        the Grader folder, but it can be anywhere.
      </li>
      <li>
        Create a <code>rubrics.csv</code> file in <code>./LT2</code>. Copy and 
        paste the rubrics from the rubrics Word file to create it.
      </li>
      <li>
        Download all student submissions to, e.g, <code>./LT2/submissions/</code>. 
      </li>
      <li>
          In case there are email submissions, download them to, for instance, 
        <code>./LT2/submissions/email/</code>.      
      <li>
        Download Grade Items for the lab test into one CSV each per section.
      </li>
    </ul>";
  private static $intro0 = "<p>The normal workflow is expected to be carried out
  by clicking on the buttons from left to right in the top frame. The buttons 
  <button class='success'><strong> &Leftarrow; Previous</strong></button> and 
  <button class='success'><strong>Next &Rightarrow;</strong></button> can also be used.
  The first 
  <button class='info'><strong>Quick Info</strong></button> button will bring up 
  this information. The <button class='success'><strong>Home&Uparrow;</strong></button> 
  button will take you back to the main menu, the root home page.</p>
  <p>The buttons in <button class='warn'>Yellow</button> are typically not 
  needed in the normal workflow, while the ones in <button class='error'>Red
  </button> are dangerous. They may delete information or files or launch
  long, cpu-intensive processes.</p> ";
  protected static $display = ["class" => "success", "title" => ""];
  protected static $studentDao;
  protected static $answerDao;
  protected static $marksDao;
  protected static $rubricDao;
  protected static $refFileDao;
  protected static $questionDao;
  protected static $subQuestionDao;
  protected static $exam;
  public function __construct()
  {
    self::head();
    self::error("Attempt to instantiate static class: " . static::$class);
    self::foot();
    die;
  }
  public static function init()
  {
    $parent = realpath(__DIR__ . "/..");
    self::$graderPath = substr($parent, strlen($_SERVER['DOCUMENT_ROOT']));
    // If the display title is not set, we are doing index.php, meaning
    // it's too soon to create the DAOs
    $dao = !empty(static::$display["title"]);
    // die(static::$display["title"]);
    if ($dao) {
      self::$studentDao = new StudentDAO();
      self::$answerDao = new AnswerDAO();
      self::$marksDao = new MarksDAO();
      self::$rubricDao = new RubricDAO();
      self::$refFileDao = new RefFileDAO();
      self::$questionDao = new QuestionDAO();
      self::$subQuestionDao = new SubQuestionDAO();
      self::$exam = new Exam();
    }
  }
  /**
   * Main landing page: Inherited by subpages (in CFG, Exam, Grader, Report)
   * @return void
   */
  public static function index()
  {
    $frame = array_keys($_GET);
    if (empty($frame)) {
      self::head();
      $dbHost = "localhost";
      $db = new DB($dbHost, CFG::$dbUser, CFG::$dbPass, CFG::$dbName);
      $error = $db->getError();
      if ($error) {
        self::head();
        self::error($error);
        self::foot();
      } else if (!$db->tablesExist(CFG::$dbPrefix)) {
        self::head();
        self::warn("No tables in the DB! Please run the DB setup script again. 
        go to <strong>Setup and Configuration</strong> and set it up.");
        self::foot();
      }
      $top = static::top();
      echo "<iframe name='top' srcdoc='$top' id='top' 
      onload='resizeIframe(this);'
      style='border:none;width:96vw;height:90px;' ></iframe>\n";
      echo self::mkFrame("left");
      echo self::mkFrame("right");
      self::foot();
    } else {
      $action = array_keys($_GET)[0];
      if ($action != 'do') {
        xdebug_print_function_stack(
          "Should not get here!",
          XDEBUG_STACK_NO_DESC
        );
      }
      static::$action();
    }
  }
  protected static function mkFrame($side = 'left')
  {
    $srcdoc = htmlentities(self::quickInfo($side), ENT_QUOTES);
    $str = "<iframe name='$side' srcdoc='$srcdoc'
    style='border:none;float:left;width:48vw;height:90vh;'></iframe>\n";
    return $str;
  }
  public static function quickInfo($side)
  {
    $str = self::head(true);
    if ($side == 'left') {
      $str0 = static::$intro;
      if (__CLASS__ != static::$class) {
        $str0 .= self::say(self::$intro0, 'plain', true);
      }
    } elseif ($side = 'right') {
      $actions = static::$actions;
      $actionProperties = static::$actionProperties;
      $actionDesc = static::$actionDesc;
      $str0 = "<h4>Action Buttons</h4>";
      $str0 .= "<table cellpadding='15'>";
      foreach ($actions as $key => $button) {
        $desc = "";
        if (array_key_exists($key, $actionDesc)) {
          $desc = $actionDesc[$key];
        }
        $class = "plain";
        if (array_key_exists($key, $actionProperties)) {
          $properties = $actionProperties[$key];
          if (array_key_exists('class', $properties)) {
            $class = $properties['class'];
          }
        }
        $str0 .= "<tr>
          <td width='20%'><button class='$class'>$button</button></td>
          <td>$desc</td>
          </tr>";
      }
      $str0 .= "</table>";
    }
    $display = static::$display["class"];
    $str .= self::$display($str0, true);
    $str .= self::foot(true);
    return "<div id='div$side'>$str</div>";
  }
  private static function top()
  {
    if (empty($_POST)) {
      $top = self::head(true);
      $title = static::$display['title'];
      if ($title) {
        $title = ": $title";
      }
      $what = "<div style='text-align:center;font-size:120%;font-weight:bold'>"
        . CFG::$examLongName . $title . "</div>";
      $actions = static::$actions;
      $what .= static::mkTop($actions);
      $display = static::$display["class"];
      $top .= self::$display($what, true);
      $top .= self::foot(true);
      $top = htmlentities($top, ENT_QUOTES);

      // $top = str_replace([ '"', '&' ], [ '&quot;', '&amp;amp;' ], $top);

      return $top;
    }
  }
  private static function mkButtons()
  {
    $buttons = [];
    $next = $prev = "";
    if (empty(static::$display["title"])) {
      return $buttons;
    }
    $action = basename($_SERVER['REQUEST_URI'], ".php");
    $actions = array_keys(self::$actions);
    $nActions = count($actions);
    for ($i = 0; $i < $nActions; $i++) {
      if ($action == $actions[$i]) {
        if ($i > 0) {
          $prev = $actions[$i - 1];
        }
        if ($i < $nActions - 1) {
          $next = $actions[$i + 1];
        }
        break;
      }
    }
    if ($prev) {
      $prevText = self::$actions[$prev];
      $buttons['prev'] = "<td>
      <a href='$prev.php' target='_parent'><button class='success'><strong>
      &Leftarrow; $prevText</strong></button></a></td>";
    }
    if ($next) {
      $nextText = self::$actions[$next];
      $buttons['next'] = "<td>
      <a href='$next.php' target='_parent'><button class='success'><strong>
      $nextText &Rightarrow;</strong></button></a></td>";
    }
    return $buttons;
  }
  private static function mkTop($actions)
  {
    $back = !empty(static::$display["title"]);
    $what = "<table align='center'><tr>";
    $buttons = self::mkButtons();
    if ($back) {
      $target = "target='left'";
    } else {
      $target = "target='_parent'";
    }
    $method = "method='post'";
    if ($back) {
      if (array_key_exists('prev', $buttons)) {
        $what .= $buttons['prev'];
      }
      $what .= "
      <td>
      <a href='' target='_parent'>
      <button type='submit' class='info'>
      <strong>Quick Info</strong>
      </button>
      </a>
      </td>";
    }
    foreach ($actions as $name => $value) {
      $action = "action='?do'";
      $class = "class='plain'";
      $force = $clearFrames = "";
      if (array_key_exists($name, static::$actionProperties)) {
        $actionProperties = static::$actionProperties[$name];
        if (array_key_exists('target', $actionProperties)) {
          $target = $actionProperties['target'];
          $target = "target='$target'";
        }
        if (array_key_exists('class', $actionProperties)) {
          $class = $actionProperties['class'];
          $class = "class='$class'";
        }
        if (array_key_exists('clear', $actionProperties)) {
          $clearFrames = "onclick='window.parent.right.document.write(); 
          window.parent.left.document.write();'";
        }
        if (array_key_exists('force', $actionProperties)) {
          $force = "<input type='hidden' name='force' value='force'>";
        }
      }
      $what .= "
      <td>
      <form $action $method $target style='text-align:center;'>
      <input type='hidden' name='do' value='$name'>
      $force
      <button type='submit' $class $clearFrames>
      $value
      </button>
      </form>
      </td>";
    }
    if ($back) {
      $what .= "
      <td>
      <a href='..' target='_parent'>
      <button type='submit' class='success'>
      <strong>Home &Uparrow;</strong>
      </button>
      </a>
      </td>";
      if (array_key_exists('next', $buttons)) {
        $what .= $buttons['next'];
        ;
      }
    }
    $what .= "</tr></table>";
    return $what;
  }
  // HTML Helper functions
  public static function head(
    $toStr = false,
    $title = "Grader by Manoj",
    $refresh = ''
  ) {
    $uiPath = self::$graderPath . "/ui";
    $css = "$uiPath/css/error-bar.css";
    $js = "$uiPath/js/utils.js";
    if (!empty($refresh)) {
      $refresh = "<meta http-equiv='refresh' content='$refresh'>";
    }
    $str = "<!DOCTYPE html>
    <html>
      <head>
        <title>$title</title>
        <link href='$css' rel='stylesheet'>
        $refresh
        <script src='$js'></script>
      </head>
      <body>
      ";
    if ($toStr) {
      return $str;
    } else {
      echo $str;
    }
  }
  public static function codeHead($title = "Code Listing by Manoj")
  {
    $path = ".";
    $str = "<!DOCTYPE html>
    <html>
      <head>
        <title>$title</title>
        <link rel='stylesheet' href='$path/highlight/styles/agate.min.css'>
        <script src='$path/highlight/highlight.min.js'></script>
        <script>hljs.highlightAll();</script>
      </head>
      <body>
      ";
    echo $str;
  }
  public static function foot($toStr = false)
  {
    $str = "  </body>
    </html>";
    if ($toStr) {
      return $str;
    } else {
      echo $str;
    }
  }
  public static function say($what, $class = 'plain', $toStr = false, $showIcon = false)
  {
    if ($showIcon) {
      $icon = self::$ico[$class];
      $icon = "<i class='ico'>$icon</i>";
    } else {
      $icon = "";
    }
    $str = "    <div class='bar  $class'>
    $icon $what
  </div>
  ";
    if ($toStr) {
      return $str;
    } else {
      echo $str;
    }
  }
  public static function getMsgTypes()
  {
    return array_keys(self::$ico);
  }
  public static function error($what, $toStr = false)
  {
    return self::say($what, $class = "error", $toStr);
  }
  public static function warn($what, $toStr = false)
  {
    return self::say($what, $class = "warn", $toStr);
  }
  public static function info($what, $toStr = false)
  {
    return self::say($what, $class = "info", $toStr);
  }
  public static function success($what, $toStr = false)
  {
    return self::say($what, $class = "success", $toStr);
  }
  public static function plain($what, $toStr = false)
  {
    return self::say($what, $class = "plain", $toStr);
  }
  public function print($toStr = true)
  {
    $str = "";
    $msgTypes = self::getMsgTypes();
    foreach ($msgTypes as $fun) {
      if ($this->$fun) {
        $str .= self::$fun($this->$fun, true);
      }
    }
    return $str;
  }
  private static function do($method = '')
  {
    $class = static::$class;
    if (array_key_exists('do', $_POST)) {
      $action = $_POST['do'];
    } else if (array_key_exists('do', $_GET)) {
      $action = $_GET['do'];
    } else {
      self::head();
      self::error("Got an unknown static dispatch request: $class::$method()!<br>
      Dispatched to HT::do(), but failed.");
      var_dump("POST:", $_POST, "GET:", $_GET);
      xdebug_print_function_stack(
        "Should not be here!",
        XDEBUG_STACK_NO_DESC
      );
      self::foot();
      exit();
    }
    if (method_exists($class, $action)) {
      $class::$action();
      exit();
    } else { // In the HT class, it's just a redirection to subPages
      header("Location: ui/$action.php");
      exit();
    }
  }
  public static function __callStatic($method, $args)
  { // Handle static dispatch request: $class::$method()
    static::do($method);
    exit();
  }
}
