<?php
class Question extends HT
{
    private $num;
    private $marks = 0;
    private $subQuestions = [];
    /**
     * Question out of the question number and assoc array of subquestions
     *
     * @param int $num: Question number
     * @param []|float $marks: Subquestion => marks
     */
    public function __construct($num, $marks)
    {
        $this->num = $num;
        if (is_array($marks)) {
            foreach ($marks as $subQuestionName => $subQuestionMarks) {
                if ($this->isSubquestion($subQuestionName)) {
                    $this->marks += $subQuestionMarks;
                    $this->subQuestions[$subQuestionName] =
                        new SubQuestion($subQuestionName, $subQuestionMarks, $num);
                }
            }
        } else {
            $this->marks = $marks;
        }
    }
    public function render($student)
    {
        $srcDoc = HT::head(true);
        foreach ($this->subQuestions as $subQuestion) {
            $srcDoc .= $subQuestion->render($student);
        }
        $srcDoc .= HT::foot(true);
        return $srcDoc;
    }
    public function print($toStr = true)
    {
        $str = "<h3>&emsp;Question: $this->num, &emsp;Marks:$this->marks</h3>";
        foreach ($this->subQuestions as $subQuestion) {
            $str .= $subQuestion->print();
        }
        return $str;
    }
    public function view()
    {

    }
    private function isSubquestion($sqName)
    {
        return stripos($sqName, "q$this->num") === 0;
    }
    /**
     * @return mixed
     */
    public function getSubQuestions()
    {
        return $this->subQuestions;
    }
    /**
     * @return mixed
     */
    public function getMarks()
    {
        return $this->marks;
    }
    /**
     * @return mixed
     */
    public function getNum()
    {
        return $this->num;
    }

	/**
	 * @param mixed $subQuestions 
	 * @return self
	 */
	public function setSubQuestions($subQuestions): self {
		$this->subQuestions = $subQuestions;
		return $this;
	}
}