<?php
class Report extends HT
{
    static $algo = "sha1";
    static $simLimit = 98;
    protected static $actions = [
        // 'stats' => 'Marks Statistics',
        'load' => 'Load Student List',
        'start' => 'Build Database Tables',
    ];
    protected static $actionProperties = [
        'start' => [
            'class' => 'error',
            'target' => 'left'
        ],
        'stop' => [
            'class' => 'error',
            'target' => 'left'
        ]
    ];
    protected static $actionDesc = [
        'stats' => 'Marks Statistics: Not implemented yet.',
        'load' => 'Load Student List for in-depth analysis for copying/cheating
        attempts.',
        'start' => 'Build FBI Table: The cheating investigation is done by
        comparing each submitted file against all other files from all other 
        students. Launch this (one-time) processing to create the necessary
        database tables. It will take a while. But it can be interrupted and 
        restarted without losing what is computed.',
    ];
    protected static $intro = "<h4>Reports and Investigation</h4>
    <h5> Statistics and Detection of Academic Dishonesty</h5>
  <p> Grader can help you detect cheating attempts using heuristics: Missing or 
  wrong emails in the files, content similarity, number of submitted files that
  look similar to the same student etc.</p>";
    protected static $display = [
        "class" => "warn",
        "title" => "Investigate"
    ];
    protected static $class = __CLASS__;
    private static $fbiFile = 'temp.fbi.html';
    private static $cutoffs = [
        100 =>
        [
            'class' => 'error',
            'msg' => [
                'solution' => 'Question leak likely!',
                'resource' => 'Question not attempted',
                'other' => 'Cheating highly likely!'
            ]
        ],
        98 =>
        [
            'class' => 'error',
            'msg' => [
                'solution' => 'Possible leak!',
                'other' => 'Possible cheating!'
            ]
        ],
        90 => [
            'class' => 'warn',
            'msg' => [
                'other' => 'Check for cheating!'
            ]
        ],
        80 => [
            'class' => 'plain',
            'msg' => [
                'other' => 'Slight chance of cheating'
            ]
        ],
        0 => [
            'class' => 'success',
            'msg' => [
                'other' => 'All good'
            ]
        ],
    ];
    private static $maxExecutionTime = 3600;
    public static function hash($file, $raw = true)
    {
        if ($raw) {
            $data = file_get_contents($file);
        } else {
            $data = self::trimFile($file);
        }
        return hash(self::$algo, $data);
    }
    public static function trimFile($file)
    {
        $lines = file($file);
        $data = "";
        for ($i = 0; $i < count($lines); $i++) {
            $data .= self::trim($lines[$i]);
        }
        return $data;
    }
    private static function trim($line)
    {
        $trimmed = strtolower(preg_replace("/\s+/", "", $line));
        if (strpos($trimmed, "user:") === 0) {
            $trimmed = "";
        }
        if (strpos($trimmed, "name:") === 0) {
            $trimmed = "";
        }
        if (strpos($trimmed, "email:") === 0) {
            $trimmed = "";
        }
        if (strpos($trimmed, "<?php") === 0) {
            $trimmed = "";
        }
        if (strpos($trimmed, "?>") === 0) {
            $trimmed = "";
        }
        return $trimmed;
    }
    public static function similarity($f1, $f2)
    {
        $d1 = self::trimFile($f1);
        $d2 = self::trimFile($f2);
        $similarity = 0.0;
        similar_text($d1, $d2, $similarity);
        return $similarity;
    }
    public static function getSimilarity($answer, $reference)
    {
        // var_dump(date("h:m:ss"), $answer->getFileName());
        $sim = [];
        if ($answer->getHash0() == $reference->getHash0()) {
            $sim[0] = 100;
        }
        if ($answer->getHash1() == $reference->getHash1()) {
            $sim[1] = 100;
        }
        if (!isset($sim[0])) {
            $similarity = 0.0;
            similar_text(
                $answer->getFullText(),
                $reference->getFullText(),
                $similarity
            );
            $sim[0] = round($similarity, 2);
        }
        if (!isset($sim[1])) {
            $sim[1] = round(self::similarity(
                $answer->getFileName(),
                $reference->getFileName()
            ), 2);
        }
        return $sim;
    }
    protected static function showSummary()
    {
        self::init();
        $email = $_POST['email'];
        $student = self::$studentDao->getByEmail($email)[0];
        $studentName = $student->getName();
        $srcDoc = HT::head(true);
        $answers = self::$answerDao->getByObject($student);
        $table1 = "<table class='grader' align='center'><tr>
        <th colspan='100%' style='text-align:center;font-size:120%;'>
        Summary: $studentName [$email]</th></tr>
        <tr><th>Suspect (email)</th><th>Number of Repeats</th>
        <th>Max. Similarity</th><th>Comment</th></tr>";
        $table2 = "<table class='grader' align='center'><tr>
        <th colspan='100%' style='text-align:center;font-size:120%;'>
        Details: $studentName [$email]</th></tr>
        <tr><th>Subquestion</th><th>File Name</th>
        <th>Suspect (email)</th><th>Similarity<br>Sim Resource</th><th>Comment</th></tr>";
        if (count($answers) > 0) { // Student attempted
            $allEmails = [];
            $allSims = [];
            $simRes = [];
            foreach ($answers as $answer) {
                $shortFileName = $answer->getShortFileName();
                if (!self::$refFileDao->isGradable($shortFileName)) {
                    continue;
                }
                $resource = self::$refFileDao->getRefFile($shortFileName, 'resources');
                if ($resource->getSim() > self::$simLimit) {
                    continue;
                }
                $simRes[$shortFileName] = max(Report::getSimilarity($answer, $resource));
                if ($simRes[$shortFileName] < 100) {
                    $row = self::getFBI($student, $answer);
                    if (count($row) == 0) {
                        $srcDoc .= HT::error("Student " . $student->getName() .
                            " not processed yet. No info in the Database!", true);
                        $srcDoc .= HT::foot(true);
                        echo $srcDoc;
                        return;
                    }
                    $r = $row[0];
                    for ($i = 0; $i < 5; $i++) {
                        $key = "sim$i";
                        $otherEmail = $r[$key . "_email"];
                        if (array_key_exists($otherEmail, $allSims)) {
                            if ($allSims[$otherEmail] < $r[$key]) { // Get max value
                                $allSims[$otherEmail] = $r[$key];
                            }
                        } else {
                            $allSims[$otherEmail] = $r[$key];
                        }
                        $allEmails[] = $otherEmail;
                    }
                }
            }
            // sort($allEmails);
            // ksort($allSims);
            // var_dump($suspects, $allEmails, $allSims);
            $suspects = array_count_values($allEmails);
            arsort($suspects);
            foreach ($suspects as $otherEmail => $repeats) {
                if ($repeats < 2) {
                    continue;
                }
                $maxSim = $allSims[$otherEmail];
                if ($maxSim < 80) {
                    continue;
                }
                [$class, $comment] = self::simToMsg($maxSim, 'other');
                $table1 .= "<tr class='$class'><td>$otherEmail</td><td>$repeats</td>
                <td>$maxSim</td><td>$comment</td></tr>";
                foreach ($answers as $answer) {
                    $shortFileName = $answer->getShortFileName();
                    if (!self::$refFileDao->isGradable($shortFileName)) {
                        continue;
                    }
                    $resource = self::$refFileDao->getRefFile($shortFileName, 'resources');
                    if ($resource->getSim() > self::$simLimit) {
                        continue;
                    }
                    if ($simRes[$shortFileName] > self::$simLimit) {
                        continue;
                    }
                    $subQuestionName = $answer->getSubQuestionName();
                    $sim = self::getSimFromDB([
                        'email1' => $email,
                        'email2' => $otherEmail,
                        'short_filename' => $shortFileName
                    ]);
                    [$class, $comment] = self::simToMsg($sim, 'other');
                    $table2 .= "<tr class='$class'><td>$subQuestionName</td>
                    <td>$shortFileName</td><td>$otherEmail</td>
                    <td>$sim<br>$simRes[$shortFileName]</td><td>$comment</td></tr>";
                }
            }
        } else {
            $table1 .= "<tr><td colspan='100%'>Not Attempted</td></tr>";
            $table2 .= "<tr><td colspan='100%'>Not Attempted</td></tr>";
        }
        $closeTable = "</table>";
        $table1 .= $closeTable;
        $table2 .= $closeTable;
        $srcDoc .= HT::warn($table1, true);
        $srcDoc .= HT::warn($table2, true);
        $srcDoc .= HT::foot(true);
        echo $srcDoc;
        return $srcDoc;
    }
    // Dispatch handlers
    protected static function stats()
    {
        self::init();
        $exam = self::$exam;
        $exam->stats();
    }
    protected static function load()
    {
        self::init();
        $exam = self::$exam;
        HT::head();
        foreach ($exam->getSections() as $section) {
            HT::success("<h3>Section: " . $section->getName() . "</h3>");
            foreach ($section->getStudents() as $student) {
                if ($student->getStatus() != "Absent") {
                    self::listStudent($student);
                }
            }
        }
        HT::foot();
    }
    private static function wait($iframe)
    { // iframe = right, or right.right1 etc.
        $js = "onclick='window.parent.$iframe.document.write(\"<h1>Please wait!</h1>\");'";
        return $js;
    }
    private static function listStudent($student)
    {
        $nStudents = self::$studentDao->getNStudents();
        $email = $student->getEmail();
        $iStudent = self::$studentDao->getIStudent($email);
        // $msg = "[Section: " . $student->getSection() . " Email: " .
        //     $student->getEmail() . "]";
        $msg = "[" . $student->getEmail() . "]";
        $wait = self::wait("right");
        $what = "<strong>{$student->getName()}</strong>&emsp;<small>$msg&ensp;
            ($iStudent of " . $nStudents . ")</small>
            <table align='right'><tr>
            <td><form method='post' action='?do' target='right'>
            <input type='submit' value='Prepare' $wait>
            <input type='hidden' name='do' value='processOne'>
            <input type='hidden' name='email' value='$email'>
            </form></td>
            <td><form method='post' action='?do' target='right'>
            <input type='submit' value='Investigate'>
            <input type='hidden' name='do' value='investigate'>
            <input type='hidden' name='email' value='$email'>
            </form></td>
            <td><form method='post' action='?do' target='right'>
            <input type='submit' value='Summary'>
            <input type='hidden' name='do' value='showSummary'>
            <input type='hidden' name='email' value='$email'>
            </form></td></tr></table>";
        HT::warn($what);
    }
    private static function putFBI($student, $answer, $similarity, $fileEmail)
    {
        $row = [];
        $row['student_email'] = $student->getEmail();
        $row['subquestion_name'] = $answer->getSubQuestionName();
        $row['filename'] = $answer->getFileName();
        $row['hash0'] = $answer->getHash0();
        $row['hash1'] = $answer->getHash1();
        $row['file_email'] = $fileEmail;
        $file = $answer->getFileName();
        $basename = basename($file);
        $resource = self::$refFileDao->getRefFile($basename, 'resources');
        $solution = self::$refFileDao->getRefFile($basename, 'solutions');
        $row['sim_solution'] = max(self::getSimilarity($answer, $solution));
        $row['sim_resource'] = max(self::getSimilarity($answer, $resource));
        $keys = array_keys($similarity);
        $bound = min(5, count($keys));
        for ($i = 0; $i < $bound; $i++) {
            $row["sim$i"] = $similarity[$keys[$i]];
            $row["sim$i" . "_email"] = $keys[$i];
        }
        $table = CFG::mkTableName('fbi');
        DB::$db->update($table, array_keys($row), $row);
    }
    private static function getFBI($student, $answer)
    {
        $where = [];
        $where['student_email'] = $student->getEmail();
        $where['subquestion_name'] = $answer->getSubQuestionName();
        $where['filename'] = $answer->getFileName();
        $table = CFG::mkTableName('fbi');
        $row = DB::$db->get($table, "*", $where);
        return $row;
    }
    private static function getSimFromDB($where)
    {
        $table = CFG::mkTableName('fbi_matrix');
        $row = DB::$db->get($table, "sim", $where);
        if (empty($row)) {
            extract($where);
            $where['email1'] = $email2;
            $where['email2'] = $email1;
            $row = DB::$db->get($table, "sim", $where);
        }
        if (empty($row)) {
            return false;
        }
        return $row[0]['sim'];
    }
    private static function putSimToDB($toDB)
    {
        $table = CFG::mkTableName('fbi_matrix');
        DB::$db->update($table, array_keys($toDB), $toDB);
    }
    protected static function start()
    {
        $fbiFile = self::$fbiFile;
        $fbi = HT::head(true, "Status Watcher");
        $wait = self::wait("right");
        $fbi .= HT::warn(
            "<p>Building the FBI table for the first time will take a long time. 
            Processing by section will be faster because time is proportional 
            to the square of the number of sections processed. But comparing
            the answers from all students may reveal organized cheating 
            attempts.</p>
            <p>After launching the process, you can monitor the progress, which 
            will run in the background as long as the right frame is open.</p>
            <p>You stop the process any time. Relaunching it will pick up from 
            where you last left off.</p>
            <table align='center'><tr>
            <td><form target='right' method='post' action='report.php?do'>
            <input type='hidden' name='do' value='process'>
            <label>
            <input type='checkbox' name='sectioned' checked>Process By Section?
            </label>&nbsp;
            <button type='submit' class='error' $wait>
            Proceed to Build FBI Table?
            </button></form></td>
            <td><form target='left' action=''>
            &emsp;<button type='submit' class='success'>
            Monitor Status
            </button></form></td>
            </tr></table>",
            true
        );
        $fbi .= HT::foot(true);
        file_put_contents($fbiFile, $fbi);
        header("Location: $fbiFile");
    }
    protected static function stop()
    {
        $fbiFile = self::$fbiFile;
        $heystack = file_get_contents($fbiFile);
        $needle = "<body>";
        $start = stripos($heystack, $needle) + strlen($needle);
        $needle = "<table";
        $end = stripos($heystack, $needle);
        $body = substr($heystack, $start, $end - $start);
        $fbi = HT::head(true);
        $fbi .= HT::warn($body, true);
        $fbi .= HT::error("FBI Database Processing stopped.
        Run it again to finish preparing the FBI database.", true);
        $fbi .= HT::foot(true);
        file_put_contents($fbiFile, $fbi);
        echo $fbi;
        // unlink($fbiFile);
    }
    /*
    This algorithm/design has been improved: (=> V3)
    1.  We have a new temp table: fbi_matrix with columns
    subquestion_name, email1, email2, sim
    2.  Before processing each new student (in processOne()),
    we look for the similarity in this table (email2, email1)
    3.  We do the computation only if needed.
    */
    protected static function process()
    {
        function rutime($ru, $rus, $index)
        {
            return ($ru["ru_$index.tv_sec"] * 1000 +
                intval($ru["ru_$index.tv_usec"] / 1000))
                - ($rus["ru_$index.tv_sec"] * 1000 +
                    intval($rus["ru_$index.tv_usec"] / 1000));
        }
        self::init();
        set_time_limit((int) self::$maxExecutionTime);
        date_default_timezone_set('Asia/Singapore');
        $start = microtime(true);
        $cpuStart = getrusage();
        $date = date('m/d/Y h:i:s a');
        $fbi0 = "<h4>Process Monitor</h4>[$date] Processing Started<br>";
        $fbiFile = self::$fbiFile;
        $studentDao = self::$studentDao;
        $students = $studentDao->get();
        $nStudents = $studentDao->getNStudents();
        foreach ($students as $student) {
            $studentEmail = $student->getEmail();
            $iStudent = $studentDao->getIStudent($studentEmail);
            $fbi = HT::head(true, "Status Watcher", '5');
            $date = date('m/d/Y h:i:s a');
            $end = microtime(true);
            $elapsed = gmdate("H:i:s", round($end - $start));
            $cpuNow = getrusage();
            $usedCPU = rutime($cpuNow, $cpuStart, 'utime') / 1000;
            $usedCPU1 = gmdate("H:i:s", round($usedCPU));
            $fbi1 = "[$date] Processing Student $iStudent of $nStudents <br>
            [CPU/Elapsed Time: $usedCPU1/$elapsed]<br>";
            if ($iStudent % 10 == 1) {
                $fbi0 .= $fbi1;
            }
            $fbi2 = "<table align='right'><tr>
            <td><form target='right' method='post' action='report.php?do'>
            <input type='hidden' name='do' value='stop'>
            <button type='submit' class='error'>
            Stop Processing
            </button></form></td>
            </tr></table><br>";
            $fbi .= HT::info("<code>$fbi0</code>" .
            HT::warn("<code>$fbi1</code>", true), true);
            $fbi .= HT::foot(true);
            if (!file_exists($fbiFile)) { // Interrupted by self::stop()
                break;
            } else {
                file_put_contents($fbiFile, $fbi);
            }
            // Quit 15 secs before execution time limit
            if ($usedCPU > self::$maxExecutionTime - 15) {
                break;
            }
            self::processOne($studentEmail, true);
        }
        $fbi = HT::head(true, "Status Watcher");
        if ($iStudent == $nStudents) {
            $fbi .= HT::success(
                "<code>$fbi0 $fbi1<br>
                Processed all $nStudents students. FBI Database is 
                ready!</code>",
                true
            );
        } else {
            $fbi = HT::warn(
                "<code>$fbi0 $fbi1<br>
                Processed $iStudent of $nStudents students. Run it again to 
                finish preparing the FBI database.</code>",
                true
            );
        }
        $fbi .= HT::foot(true);
        file_put_contents($fbiFile, $fbi);
        echo $fbi;
    }
    private static function getEmail($fullText)
    {
        $lines = preg_split('/\n|\r/', $fullText, -1, PREG_SPLIT_NO_EMPTY);
        foreach ($lines as $line) {
            if (stripos($line, "email:") !== false) {
                $email = trim(explode(":", $line)[1]);
                return $email;
            }
        }
    }
    protected static function processOne($studentEmail = '', $toStr = true)
    {
        self::init();
        if (empty($studentEmail)) {
            $studentEmail = $_POST['email'];
        }
        if (ini_get('max_execution_time') < 60) {
            ini_set('max_execution_time', 60);
        }
        $studentDao = self::$studentDao;
        $student = $studentDao->getByEmail($studentEmail)[0];
        $submissionFolder = $student->getAnswerFolder();
        $where = [];
        if (isset($_POST['sectioned'])) {
            $where = ['section' => $student->getSection()];
        }
        $others = $studentDao->get($where);
        $exam = self::$exam;
        $answerDao = self::$answerDao;
        $str = HT::head($toStr);
        $str0 = "";
        $toDB = [];
        $toDB['email1'] = $studentEmail;
        foreach ($exam->getQuestions() as $question) {
            $str0 .= HT::warn("Student: " . $student->getName() .
                ", Question " . $question->getNum(), $toStr);
            foreach ($question->getSubQuestions() as $subQuestion) {
                $subQuestionName = $subQuestion->getName();
                $toDB['subquestion_name'] = $subQuestionName;
                $similarity = [];
                $answers = $answerDao->getByObject($student, $subQuestion);
                if (count($answers) == 0) {
                    continue;
                }
                foreach ($answers as $answer) {
                    $row = self::getFBI($student, $answer);
                    $shortFileName = $answer->getShortFileName();
                    if (!self::$refFileDao->isGradable($shortFileName)) {
                        continue;
                    }
                    $resource = self::$refFileDao->getRefFile($shortFileName, 'resources');
                    if ($resource->getSim() > self::$simLimit) {
                        continue;
                    }
                    if (count($row) > 0) {
                        $str0 .= HT::success("&emsp;&rarr; SubQuestion 
                        $subQuestionName [$shortFileName] read from DB.", $toStr);
                        continue;
                    } else {
                        $str0 .= HT::error(
                            "&emsp;&rarr; SubQuestion 
                        $subQuestionName [$shortFileName] building: Will take a minute.",
                            $toStr
                        );
                    }
                    $toDB['short_filename'] = $shortFileName;
                    $fileEmail = self::getEmail($answer->getFullText());
                    foreach ($others as $other) {
                        $otherEmail = $other->getEmail();
                        if ($otherEmail == $studentEmail) {
                            continue;
                        }
                        $otherAnswer = $answerDao->getByObject(
                            $other,
                            $subQuestion,
                            $answer
                        );
                        $count = count($otherAnswer);
                        if ($count == 0) {
                            continue;
                        }
                        if ($count > 1) {
                            $str0 .= HT::error(
                                "&emsp;&rarr; Got too many answers ($count) from "
                                . $other->getName()
                            );
                        }
                        $toDB['email2'] = $otherEmail;
                        $fromDB = self::getSimFromDB($toDB);
                        if ($fromDB === false) {
                            $otherAnswer = reset($otherAnswer);
                            $sim = max(self::getSimilarity($otherAnswer, $answer));
                            $similarity[$otherEmail] = $sim;
                            $toDB['sim'] = $sim;
                            self::putSimToDB($toDB);
                        } else {
                            $similarity[$otherEmail] = $fromDB;
                        }
                    }
                    arsort($similarity, SORT_NUMERIC);
                    self::putFBI($student, $answer, $similarity, $fileEmail);
                }
            }
        }
        $str .= HT::info($str0);
        $str .= HT::foot($toStr);
        echo $str;
    }
    private static function simToMsg($sim, $type)
    {
        $cutoffs = self::$cutoffs;
        foreach ($cutoffs as $cutoff => $details) {
            if ($sim >= $cutoff) {
                break;
            }
        }
        $class = $details['class'];
        $msg = "All good";
        if (array_key_exists($type, $details['msg'])) {
            $msg = $details['msg'][$type];
        }
        return [$class, $msg];
    }
    private static function investigateRow($row)
    {
        $r = $row[0];
        $tr = "<tr><th>Issue</th><th>Details</th><th>Comments</th></tr>";
        if (empty($r['file_email'])) {
            $tr .= "<tr class='warn'><td>File Email missing</td>
            <td>No email found in the file</td><td></td></tr>";
        } elseif (stripos($r['file_email'], $r['student_email']) === false) {
            $tr .= "<tr class='error'><td>Email mismatch</td>
            <td>Student: {$r['student_email']} != File: {$r['file_email']}</td>
            <td>Possible Cheating</td></tr>";
        } else {
            $tr .= "<tr class='success'><td>Emails match</td>
            <td> Student and file emails: {$r['student_email']}</td>
            <td>All Good</td></tr>";
        }
        $sim = $r['sim_solution'];
        [$class, $msg] = self::simToMsg($sim, 'solution');
        $tr .= "<tr class='$class'><td>Similarity to Solution</td>
        <td>Similarity = $sim</td>
        <td>$msg</td></tr>";
        $sim = $r['sim_resource'];
        [$class, $msg] = self::simToMsg($sim, 'resource');
        if ($sim > 99.999) {
            $tr .= "<tr class='$class'><td>Similarity to Resource</td>
            <td>Similarity = $sim</td>
            <td>$msg</td></tr>";
        } else {
            for ($i = 0; $i < 5; $i++) {
                $key = "sim$i";
                $sim = $r[$key];
                [$class, $msg] = self::simToMsg($sim, 'other');
                $other = $r[$key . "_email"];
                $tr .= "<tr class='$class'><td>Similarity to  $other </td>
                <td>Similarity = $sim</td>
                <td>$msg</td></tr>";
            }
        }
        return $tr;
    }
    protected static function investigate()
    {
        self::init();
        $toStr = true;
        $studentEmail = $_POST['email'];
        $studentDao = self::$studentDao;
        $student = $studentDao->getByEmail($studentEmail)[0];
        $exam = self::$exam;
        $answerDao = self::$answerDao;
        $str = HT::head($toStr);
        $studentName = $student->getName();
        $str0 = "";
        $str1 = "<table class='grader plain' align='center'>
        <tr><th colspan='100%' style='font-size:110%;'>
        FBI Report: $studentName</th></tr>";
        foreach ($exam->getQuestions() as $question) {
            foreach ($question->getSubQuestions() as $subQuestion) {
                $subQuestionName = $subQuestion->getName();
                $answers = $answerDao->getByObject($student, $subQuestion);
                if (count($answers) == 0) {
                    $str0 .= HT::error("&emsp;&rarr; SubQuestion 
                    $subQuestionName: Got no answers.", $toStr);
                    continue;
                }
                foreach ($answers as $answer) {
                    $shortFileName = $answer->getShortFileName();
                    if (!self::$refFileDao->isGradable($shortFileName)) {
                        continue;
                    }
                    $resource = self::$refFileDao->getRefFile($shortFileName, 'resources');
                    if ($resource->getSim() > self::$simLimit) {
                        continue;
                    }
                    $basename = basename($shortFileName);
                    $str1 .= "<tr><th colspan='100%'>
                    SubQuestion: $subQuestionName, 
                    File: <code>$basename</code></th></tr>";
                    $row = self::getFBI($student, $answer);
                    $count = count($row);
                    if ($count == 1) {
                        $str0 .= HT::success("&emsp;&rarr; SubQuestion 
                        $subQuestionName [$basename] read from DB.", $toStr);
                    } else {
                        $str0 .= HT::error("&emsp;&rarr; SubQuestion 
                        $subQuestionName [$basename]: Please process first 
                        [$count rows].", $toStr);
                        continue;
                    }
                    $str1 .= self::investigateRow($row);
                }
            }
        }
        $str1 .= "</table>";
        $str1 = HT::info($str1, true);
        // $str .= HT::warn($str0);
        $str .= HT::warn($str1);
        $str .= HT::foot($toStr);
        echo $str;
    }
}