<?php
class Section extends HT
{
    private $students = [];
    private $name;
    public function __construct($name)
    {
        $this->name = $name;
        $this->students = self::$studentDao->get(["section" => $name]);
        if (empty($this->students)) { // Make students from the CSV files
            if (!CFG::isQuiz()) { // Add students only for lab tests
                $this->mkStudents();
            }
        }
    }
    /**
     * Get students from the CSV file and prepare their solutions from their eLearn
     * uploads. 
     *
     * @return void
     */
    private function mkStudents()
    {
        $csv = CFG::getCsvFileName($this->name);
        if (!is_file($csv)) {
            HT::error("Error loading $csv");
            return;
        }
        $file = fopen($csv, 'r');
        $line = fgetcsv($file); // Skip the header
        while (($line = fgetcsv($file)) !== FALSE) {
            //$line is an array of the csv elements
            $this->students[] = new Student($line[0], $line[1], $this->name);
        }
        fclose($file);
    }
    public function grade($name = '')
    {
        HT::say("<h4>Section: $this->name</h4>");
        foreach ($this->students as $student) {
            $student->grade();
        }
    }
    public function delete($name = '')
    {
        foreach ($this->students as $student) {
            $student->delete();
        }
    }

    public function csv()
    {
        $str = "<table class='grader' width='98%'><tr><th colspan='100%'>Section: $this->name</th></tr>";
        foreach ($this->students as $student) {
            $str .= $student->csv();
        }
        $str .= "</table>";
        HT::warn($str);
    }
    public function print($toStr = true)
    {
        $str = $this->info("<strong>&emsp;Section: $this->name</strong>", $toStr);
        foreach ($this->students as $std) {
            $str .= $std->print();
        }
        return $str;
    }
    /**
     * @return mixed
     */
    public function getStudents()
    {
        return $this->students;
    }
    public function setStudents($students)
    {
        $this->students = $students;
        return $this;
    }
    /**
     * @return mixed
     */
    public function getName()
    {
        return $this->name;
    }
    public function export()
    {
        $csv = CFG::getCsvFileName($this->name);
        if (!is_file($csv)) {
            HT::error("Error loading $csv");
            return;
        }
        $exported = dirname($csv) . "/Export-" . basename($csv);
        $fileIn = fopen($csv, 'r');
        $fileOut = fopen($exported, 'w');
        if ($fileOut === false) {
            HT::error("Error opening the file $exported");
        }
        if (CFG::isQuiz()) {
            $headers = ["email", "Username"];
            foreach (CFG::$autoGradeOptions['subQuestionNames'] as $qNum) {
                $headers[] = $qNum;
            }
        } else {
            $headers = fgetcsv($fileIn); // copy the header
        }
        fputcsv($fileOut, $headers);
        foreach ($this->students as $student) {
            $line = [];
            $line[0] = "#" . $student->getEmail();
            $line[1] = $student->getName();
            $marks = $student->sumMarksBySubQuestion();
            foreach ($headers as $iCol => $header) {
                if ($iCol < 2) {
                    continue;
                }
                $line[$iCol] = "";
                foreach ($marks as $subQuestionName => $m) {
                    if (stripos($header, $subQuestionName) !== false) {
                        $line[$iCol] = $m;
                        break;
                    }
                }
            }
            if (CFG::isQuiz()) {
                fputcsv($fileOut, $line);
            } else {
                fputcsv($fileOut, $line, ",", "\"", "\\", ",#\n");
            }
        }
        fclose($fileOut);
        $fullText = file_get_contents($exported);
        // Remove quotation marks - eLearn doesn't like them
        $fullText = str_replace("\"", "", $fullText);
        file_put_contents($exported, $fullText);
        $exported = realpath($exported);
        HT::success("Exported $this->name marks to <code>$exported</code>");
    }
    public function stats()
    {
        foreach ($this->students as $student) {
        }
        HT::success("Statistics for $this->name.<br>To be implemented.");
    }
}