<?php
class Stat extends HT
{
    // Statistics
    /**
     * Summary of getStats
     * @param [] $where: Associative Array, Keys = sectionName, questionNum, 
     * subQuestionName
     * @return [] $stats: mean, min, max, median, stdev
     */
    protected static $actions = [
        'all' => 'Overall',
        'sections' => 'By Section',
        'questions' => 'By Question',
        'subquestions' => 'By Subquestion',
        'mistakes' => 'Common Mistakes'
    ];
    protected static $actionProperties = [
        'all' => [
            'target' => 'left',
            'clear' => 'yes'
        ],
        'sections' => [
            'target' => 'right'
        ],
        'questions' => [
            'target' => 'left'
        ],
        'subquestions' => [
            'target' => 'left'
        ],
        'mistakes' => [
            'target' => 'left'
        ]
    ];
    protected static $actionDesc = [
        'all' => 'Overall statistics for all your sections.',
        'sections' => 'Statistics broken down by section.',
        'questions' => 'Statistics by question, with option to drill down by 
        sections.',
        'subquestions' => 'Statistics by subquestion, with option to drill down by 
        sections.',
        'mistakes' => 'Display comments grouped by sub-questions.',
    ];
    protected static $class = __CLASS__;
    protected static $intro = "<h4>Statistics</h4>
      <p>Obvious, isn't it? This page lets you generate marks statistics.</p>";
    protected static $display = [
        "class" => "info",
        "title" => "View Grade Statistics"
    ];
    protected static function getRows($reduce = ["section" => "s.section"])
    {
        $rows = DB::$db->query(self::mkSql($reduce))->fetchAll();
        return $rows;
    }
    protected static function printStats(
        $grouped,
        $byTitle = "Grouped by",
        $toString = false,
        $drillDown = ''
    ) {
        $rows = "";
        foreach ($grouped as $by => $marks) {
            $rows .= "<tr><th>$by</th>";
            $mean = number_format(self::mean($marks), 2);
            $std = number_format(self::stdev($marks), 2);
            $median = number_format(self::median($marks), 2);
            $min = min($marks);
            $minCount = self::count($marks, $min);
            $max = max($marks);
            $maxCount = self::count($marks, $max);
            $min = number_format($min, 2) . " <small>[$minCount]</small>";
            $max = number_format($max, 2) . " <small>[$maxCount]</small>";
            $rows .= "<td>$mean</td>
            <td>$std</td>
            <td>$median</td>
            <td>$min</td>
            <td>$max</td></tr>";
        }
        if ($toString) {
            return $rows;
        }
        $rows .= "<tr><td colspan='10'><small>[count in brackets]</small></td></tr>";
        if ($drillDown) {
            $rows .= "<tr><th colspan='100%'>
            <form method='post' target='right'>
            <input type='hidden' name='do' value='$drillDown'>
            <button class='info' type='submit'>Drill-down by Section</button>
            </form></th></tr>";
        }
        $str = "<table class='grader success' align='center'>
        <tr><th>$byTitle</th><th>Mean</th><th>Std Dev</th>
        <th>Median</th><th>Min</th><th>Max</th></tr>
        $rows
        </table>";
        HT::head();
        HT::warn($str);
        HT::foot();
    }
    protected static function printStats2($grouped2, $byTitle1, $byTitle2)
    {
        $rowsGrouped = [];
        foreach ($grouped2 as $key => $grouped) {
            $rows = self::printStats($grouped, $byTitle2, true);
            $rowSpan = substr_count($rows, "<tr>");
            $rowsGrouped[] = preg_replace(
                "/<tr>/",
                "<tr><th rowspan='$rowSpan'>$key</th>",
                $rows,
                1
            );
        }

        $str = "<table class='grader success' align='center'>
        <tr><th>$byTitle1</th><th>$byTitle2</th><th>Mean</th><th>Std Dev</th>
        <th>Median</th><th>Min</th><th>Max</th></tr>" .
            implode("\n", $rowsGrouped) .
            "<tr><td colspan='10'><small>[count in brackets]</small></td></tr>" .
            "</table>";
        HT::head();
        HT::warn($str);
        HT::foot();
    }
    private static function groupBy($rows, $by = '')
    {
        $grouped = [];
        foreach ($rows as $row) {
            if (empty($by)) {
                $key = "All";
            } else {
                $key = $row[$by];
            }
            $grouped[$key][] = (float) $row['marks'];
        }
        ksort($grouped);
        return $grouped;
    }
    private static function groupBy2($rows, $by1, $by2)
    {
        $grouped = [];
        foreach ($rows as $row) {
            $key1 = $row[$by1];
            $key2 = $row[$by2];
            if (empty($grouped[$key1][$key2])) {
                $grouped[$key1][$key2] = [];
            }
            $grouped[$key1][$key2][] = (float) $row['marks'];
        }
        ksort($grouped);
        foreach ($grouped as $key => $bySubQuestion) {
            ksort($grouped[$key]);
        }
        return $grouped;
    }

    protected static function all()
    {
        $rows = self::getRows();
        $grouped = self::groupBy($rows);
        self::printStats($grouped);
    }
    protected static function sections()
    {
        $rows = self::getRows();
        $grouped = self::groupBy($rows, 'section');
        self::printStats($grouped, 'Section');
    }

    protected static function questions()
    {
        $rows = self::getRows(['question' => 'sq.question_num']);
        $grouped = self::groupBy($rows, 'question');
        self::printStats($grouped, 'Question', false, "questions2");
    }
    protected static function questions2()
    {
        $rows = self::getRows([
            'section' => 's.section',
            'question' => 'sq.question_num'
        ]);
        $grouped2 = self::groupBy2($rows, 'question', 'section');
        self::printStats2($grouped2, 'Question', 'Section');
    }
    protected static function subquestions()
    {
        $rows = self::getRows(['subquestion' => 'sq.subquestion_name']);
        $grouped = self::groupBy($rows, 'subquestion');
        self::printStats($grouped, 'Subquestion', false, "subquestions2");
    }
    protected static function subquestions2()
    {
        $rows = self::getRows([
            'section' => 's.section',
            'subquestion' => 'sq.subquestion_name'
        ]);
        $grouped2 = self::groupBy2($rows, 'subquestion', 'section');
        self::printStats2($grouped2, 'Subquestion', 'Section');
    }

    private static function mkSql($reduce)
    {
        $select = $groupBy = "";
        foreach ($reduce as $alias => $field) {
            $select .= "$field AS '$alias',";
            $groupBy .= "`$alias`, ";
        }
        $groupBy = trim($groupBy, ", ");
        $sql = "SELECT 
            s.student_email AS 'email',
            s.student_name AS 'name',
            $select
            SUM(m.marks) AS 'marks'
        FROM 
            `{dbPrefix}students` AS s
            INNER JOIN `{dbPrefix}marks` AS m ON m.student_email = s.student_email
            INNER JOIN `{dbPrefix}subquestions` AS sq ON m.subquestion_name = sq.subquestion_name
        GROUP BY
           `email`, `name`, $groupBy";
        $sql = str_replace('{dbPrefix}', CFG::$dbPrefix, $sql);
        return $sql;
    }

    public static function mistakes()
    {
        $sql = "SELECT 
            subquestion_name, comment 
        FROM 
            `{dbPrefix}answers` 
        WHERE 
            comment <> '' 
        ORDER BY 
            subquestion_name";
        $sql = str_replace('{dbPrefix}', CFG::$dbPrefix, $sql);
        $rows = DB::$db->query($sql)->fetchAll();
        $table = "<table width='80%' align='center' class='warn grader'>
                <tr><th width='20%'>Sub Question Number</th>
                <th width='80%'>Comment</th></tr>";
        foreach ($rows as $row) {
            $comment = "<pre>" . htmlentities($row["comment"]) . "</pre>";
            $table .= "<tr><td>{$row['subquestion_name']}</td>
            <td style='text-align:left'>$comment</td></tr>";
        }
        $table .= "</table><br>";
        HT::head();
        HT::info($table);
        HT::foot();
    }
    private static function count($data, $value, $tolerance = 0.01)
    {
        $count = 0;
        foreach ($data as $val) {
            if (abs($val - $value) <= $tolerance) {
                $count++;
            }
        }
        return $count;
    }
    public static function mean($data)
    {
        // calculates mean
        $n = count($data);
        if ($n == 0) {
            $mean = 0;
        } else {
            $mean = array_sum($data) / $n;
        }
        return $mean;
    }

    public static function median($data)
    {
        // calculates median
        sort($data);
        $elements = count($data);
        if (($elements % 2) == 0) {
            $i = $elements / 2;
            return (($data[$i - 1] + $data[$i]) / 2);
        } else {
            $i = ($elements - 1) / 2;
            return $data[$i];
        }
    }

    public static function range($data)
    {
        // calculates range
        return (max($data) - min($data));
    }

    public static function var($data)
    {
        // calculates sample variance
        $n = count($data);
        if ($n <= 1) {
            $var = 0;
        } else {
            $mean = self::mean($data);
            $sum = 0;
            foreach ($data as $element) {
                $sum += pow(($element - $mean), 2);
            }
            $var = $sum / ($n - 1);
        }
        return $var;
    }

    public static function varp($data)
    {
        // calculates population variance
        $n = count($data);
        if ($n <= 1) {
            $var = 0;
        } else {
            $mean = self::mean($data);
            $sum = 0;
            foreach ($data as $element) {
                $sum += pow(($element - $mean), 2);
            }
            $var = $sum / $n;
        }
        return $var;
    }

    public static function stdev($data)
    {
        // calculates sample standard deviation
        return sqrt(self::var($data));
    }

    public static function stdevp($data)
    {
        // calculates population standard deviation
        return sqrt(self::varp($data));
    }

    public static function simple_regression($x, $y)
    {
        // runs a simple linear regression on $x and $y
        // returns an associative array containing the following fields:
        // a - intercept
        // b - slope
        // s - standard error of estimate
        // r - correlation coefficient
        // r2 - coefficient of determination (r-squared)
        // cov - covariation
        // t - t-statistic
        $output = array();
        $output['a'] = 0;
        $n = min(count($x), count($y));
        $mean_x = self::mean($x);
        $mean_y = self::mean($y);
        $SS_x = 0;
        foreach ($x as $element) {
            $SS_x += pow(($element - $mean_x), 2);
        }
        $SS_y = 0;
        foreach ($y as $element) {
            $SS_y += pow(($element - $mean_y), 2);
        }
        $SS_xy = 0;
        for ($i = 0; $i < $n; $i++) {
            $SS_xy += ($x[$i] - $mean_x) * ($y[$i] - $mean_y);
        }
        $output['b'] = $SS_xy / $SS_x;
        $output['a'] = $mean_y - $output['b'] * $mean_x;
        $output['s'] = sqrt(($SS_y - $output['b'] * $SS_xy) / ($n - 2));
        $output['r'] = $SS_xy / sqrt($SS_x * $SS_y);
        $output['r2'] = pow($output['r'], 2);
        $output['cov'] = $SS_xy / ($n - 1);
        $output['t'] = $output['r'] / sqrt((1 - $output['r2']) / ($n - 2));

        return $output;
    }
}
