<?php
class SubQuestion extends HT
{
    private $name;
    private $marks;
    private $questionNum;
    private $solutions = [];
    private $resources = [];
    private $rubrics = [];
    private $got = false;
    public function __construct($name, $mark, $questionNum)
    {
        $this->name = strtolower($name);
        $this->marks = $mark;
        $this->questionNum = strtolower($questionNum);
    }

    public function getName()
    {
        return $this->name;
    }
    public function getQuestionNum()
    {
        return $this->questionNum;
    }
    public function print($toStr = true)
    {
        $str = "<strong>&emsp;&emsp;SubQuestion: $this->name
        &emsp;&emsp;Marks: $this->marks</strong><br>";
        return $str;
    }
    private static function mkPopup($fileName)
    {
        return "onclick=
        \"var h = screen.height * 0.8;
        var t = screen.height * 0.05;
        var w = screen.width * 0.4;
        var l = screen.width * 0.05;
        window.open('', '$fileName', 'width='+ w +',height=' + h + ',left='+ l +',top='+ t +',resizable=yes,scrollbars=yes,toolbar=yes,menubar=no,location=no,directories=no,status=yes'); 
          document.getElementById('$fileName').submit();
          return false;\"";
    }
    public function render($student)
    {
        $subQuestionName = $this->name;
        $resources = self::$refFileDao->getByObject($this, 'resources');
        $solutions = self::$refFileDao->getByObject($this, 'solutions');
        $srcDoc = " <div style='font-size:110%;font-weight:bold'
            title='SubQuestion::render()'>
            SubQuestion: $subQuestionName</div>";
        $tabRes = "<table title='SubQuestion::render()'>
            <tr><th colspan='100%'>Resources</th></tr><tr>";
        $tabSoln = "<table title='SubQuestion::render()'>
            <tr><th colspan='100%'>Solutions</th></tr><tr>";
        // Get the rubric and get the filename so that the right resource/solution
        // can be highlighted
        $rubrics = self::$rubricDao->getByObject($this);
        $rubricFiles = [];
        foreach ($rubrics as $rubric){
            $rubricFile = basename($rubric->getShortFileName());
            if (!in_array($rubricFile, $rubricFiles)) {
                $rubricFiles[] = $rubricFile;
            } 
        }
        // Assumes resources and solutions are sorted in the same order.
        // If not, sort them in the RefFilesDAO class
        foreach ($resources as $key => $resource) {
            $shortFIleName = $resource->getShortFileName();
            if (!self::$refFileDao->isGradable($shortFIleName)) {
                continue;
            }
            $fileName = $resource->getFileName();
            $basename = basename($fileName);
            $fileClass = "plain";
            if (in_array($basename, $rubricFiles)) {
                $fileClass = "error";
            }
            $fullText = htmlentities($resource->getFullText(), ENT_QUOTES);
            $popup = self::mkPopup($fileName);
            $tabRes .= "<td> 
            <form method='post' action='?do' 
            target='$fileName' id='$fileName'>
            <input type='submit' name='viewfile' title='$fileName' 
            value='$basename' $popup class='$fileClass'> 
            <input type='hidden' name='do' value='viewFile'>
            <input type='hidden' name='filename' value='$fileName'>
            <input type='hidden' name='fulltext' value='$fullText'>
            </form></td>
            ";
            // TODO: Investigate: How come $solnKey wasn't necessary before?
            $solnKey = str_replace("resources", "solutions", $key);
            $solution = $solutions[$solnKey];
            $fileName = $solution->getFileName();
            $basename = basename($fileName);
            $fullText = htmlentities($solution->getFullText(), ENT_QUOTES);
            $popup = self::mkPopup($fileName);
            $tabSoln .= "<td> 
            <form method='post' action='?do' 
            target='$fileName' id='$fileName'>
            <input type='submit' name='viewfile' title='$fileName' 
            value='$basename' $popup class='$fileClass'> 
            <input type='hidden' name='do' value='viewFile'>
            <input type='hidden' name='filename' value='$fileName'>
            <input type='hidden' name='fulltext' value='$fullText'>
            </form></td>
            ";
        }
        $closeTable = "</td></tr></table>";
        $tabRes .= $closeTable;
        $tabSoln .= $closeTable;
        $srcDoc .= HT::warn("<table align='center'>
        <tr><td align='center'>$tabRes</td><td>&emsp;</td></tr>
        <tr><td colspan='100%'>&nbsp;</td></tr>
        <tr><td align='center'>$tabSoln</td></tr>
        </table>", true);
        $srcDoc = HT::success($srcDoc, true);
        $rubSrc = "<form method='post' action='?do' target='right2'
        id='$subQuestionName'>
        <table width='90%' align='center' class='grader'>
        <tr><th>Award</th><th>Ref</th><th>Rubric</th></tr>";
        // $rubrics = self::$rubricDao->getByObject($this);
        $awarded = self::$marksDao->getByObject($student);
        // var_dump($awarded);
        $totAwarded = $totQuestion = 0.0;
        $showAwarded = false;
        foreach ($rubrics as $k => $rubric) {
            $step = 0.05;
            $marks = $rubric->getMarks();
            $rubricName = $rubric->getRubricName();
            $rubricText = $rubric->getRubricText();
            $min = min([$marks, 0]);
            if ($min < 0) {
                $step = -$min;
                $min = 5 * $min;
            }
            $max = max([$marks, 0]);
            $totQuestion += $max;
            $rubricText = htmlentities($rubricText, ENT_QUOTES);
            $value = "";
            foreach ($awarded as $a) {
                if ($a->getRubricName() == $rubricName) {
                    $showAwarded = true;
                    $value = $a->getMarks();
                    $totAwarded += $value;
                }
            }
            if ($value == $max) {
                $class = "success";
            } else if ($value > 0) {
                $class = "warn";
            } else if ($value === '') {
                $class = "plain";
            } else {
                $class = "error";
            }
            $rubSrc .= "<tr class='$class'><td>
            <input name='$rubricName' type='number' 
            value='$value' min='$min' max='$max' step='$step'>
            <input name='full-$rubricName' type='hidden' value='$max'>
            </td> 
            <td> $marks </td> 
            <td style='text-align:left;'> $rubricName: 
            <code>$rubricText</code></td>
            </tr>";
        }
        if ($showAwarded) {
            $rubSrc .= "<tr><th>$totAwarded</th><th>$totQuestion</th><th>";
        } else {
            $rubSrc .= "<tr><th colspan='100%'>";
        }
        $rubSrc .= "<input type='hidden' name='do' value='putMarks'>
        <input type='submit' value='Update Marks' 
        name='putMarks' align='center'>
        <input type='submit' value='Award Full Marks' 
        name='putFullMarks' align='center'>
        </th></tr></table>
        <input type='hidden' name='name' value='{$student->getName()}'>
        <input type='hidden' name='student_email' value='{$student->getEmail()}'>
        <input type='hidden' name='subquestion_name' value='$subQuestionName'>
        </form></div>";
        // $rubSrc = htmlentities($rubSrc);
        $srcDoc .= HT::success($rubSrc, true);
        $srcDoc = HT::info($srcDoc, true);
        return $srcDoc;
    }

    /**
     * @return mixed
     */
    public function getMarks()
    {
        return $this->marks;
    }
}