CREATE DATABASE IF NOT EXISTS `{dbName}`;
USE `{dbName}`;

CREATE TABLE IF NOT EXISTS `{dbPrefix}questions` (
  `id` int(2) NOT NULL AUTO_INCREMENT,
  `created` TIMESTAMP DEFAULT NOW(),
  `question_num` int(2) NOT NULL UNIQUE,
  `marks` decimal(4,2),
  PRIMARY KEY (`id`)
) AUTO_INCREMENT=1;
ALTER TABLE `{dbPrefix}questions` AUTO_INCREMENT = 1;

CREATE TABLE IF NOT EXISTS `{dbPrefix}subquestions` (
  `id` int(2) NOT NULL AUTO_INCREMENT,
  `created` TIMESTAMP DEFAULT NOW(),
  `subquestion_name` varchar(8) NOT NULL UNIQUE,
  `question_num` int(2) NOT NULL,
  `marks` decimal(4,2) NOT NULL,
  PRIMARY KEY (`id`)
) AUTO_INCREMENT=1;
ALTER TABLE `{dbPrefix}subquestions` AUTO_INCREMENT = 1;

CREATE TABLE IF NOT EXISTS `{dbPrefix}ref_files` (
  `id` int(5) NOT NULL AUTO_INCREMENT,
  `created` TIMESTAMP DEFAULT NOW(),
  `subquestion_name` varchar(8) NOT NULL,
  `filetype` varchar(16) NOT NULL CHECK (`filetype` in ('resources', 'solutions')),
  `filename` varchar(255) NOT NULL,
  `fulltext` text,
  `hash0` varchar(128) NOT NULL,
  `hash1` varchar(128) NOT NULL,
  `sim` DECIMAL(5,2),
  `gradable` boolean,
  PRIMARY KEY (`id`),
  UNIQUE KEY `ref_file` (`subquestion_name`,`filetype`, `hash0`(50))
) AUTO_INCREMENT=1;
ALTER TABLE `{dbPrefix}ref_files` AUTO_INCREMENT = 1;

CREATE TABLE IF NOT EXISTS `{dbPrefix}rubrics` (
  `id` int(3) NOT NULL AUTO_INCREMENT,
  `created` TIMESTAMP DEFAULT NOW(),
  `subquestion_name` varchar(8) NOT NULL,
  `rubric_name` varchar(8) NOT NULL UNIQUE,
  `marks` decimal(4,2) NOT NULL,
  `rubric_text` varchar(255) NOT NULL default '',
  `short_filename` varchar(255) NOT NULL,
  PRIMARY KEY (`id`)
) AUTO_INCREMENT=1;
ALTER TABLE `{dbPrefix}rubrics` AUTO_INCREMENT = 1;

CREATE TABLE IF NOT EXISTS `{dbPrefix}students` (
  `id` int(2) unsigned NOT NULL AUTO_INCREMENT,
  `created` TIMESTAMP DEFAULT NOW(),
  `student_email` varchar(128) NOT NULL UNIQUE,
  `student_name` varchar(128) NOT NULL default '',
  `section` varchar(8) NOT NULL default '',
  `status` varchar(16) NOT NULL default '',
  `zip_filename` varchar(255) NOT NULL default '',
  `comment` longtext,
  PRIMARY KEY (`id`)
) AUTO_INCREMENT=1 ;
ALTER TABLE `{dbPrefix}students` AUTO_INCREMENT = 1;

CREATE TABLE IF NOT EXISTS `{dbPrefix}answers` (
  `id` int(5) NOT NULL AUTO_INCREMENT,
  `created` TIMESTAMP DEFAULT NOW(),
  `student_email` varchar(128) NOT NULL,
  `subquestion_name` varchar(8) NOT NULL,
  `filename` varchar(255) NOT NULL,
  `fulltext` text,
  `comment` text,
  `hash0` varchar(128) NOT NULL,
  `hash1` varchar(128) NOT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `answer` (`student_email`,`subquestion_name`, `hash0`(50))
) AUTO_INCREMENT=1;
ALTER TABLE `{dbPrefix}answers` AUTO_INCREMENT = 1;

CREATE TABLE IF NOT EXISTS `{dbPrefix}marks` (
  `id` int(3) NOT NULL AUTO_INCREMENT,
  `created` TIMESTAMP DEFAULT NOW(),
  `student_email` varchar(128) NOT NULL default '',
  `subquestion_name` varchar(8) NOT NULL,
  `rubric_name` varchar(8) NOT NULL,
  `marks` decimal(4,2) NOT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `mark` (`student_email`(16),`subquestion_name`, `rubric_name`)
) AUTO_INCREMENT=1;
ALTER TABLE `{dbPrefix}marks` AUTO_INCREMENT = 1;

CREATE TABLE IF NOT EXISTS `{dbPrefix}fbi` (
  `id` int(5) NOT NULL AUTO_INCREMENT,
  `created` TIMESTAMP DEFAULT NOW(),
  `student_email` varchar(128) NOT NULL default '',
  `subquestion_name` varchar(8) NOT NULL,
  `filename` varchar(255) NOT NULL,
  `hash0` varchar(128) NOT NULL,
  `hash1` varchar(128) NOT NULL,
  `file_email` varchar(128) NOT NULL default '',
  `sim_solution` DECIMAL(5,2),
  `sim_resource` DECIMAL(5,2),
  `sim0` DECIMAL(5,2),
  `sim0_email` varchar(128) NOT NULL default '',
  `sim1` DECIMAL(5,2),
  `sim1_email` varchar(128) NOT NULL default '',
  `sim2` DECIMAL(5,2),
  `sim2_email` varchar(128) NOT NULL default '',
  `sim3` DECIMAL(5,2),
  `sim3_email` varchar(128) NOT NULL default '',
  `sim4` DECIMAL(5,2),
  `sim4_email` varchar(128) NOT NULL default '',
  PRIMARY KEY (`id`),
  UNIQUE KEY `fbi` (`student_email`(16), `subquestion_name`, `hash0`(50))
) AUTO_INCREMENT=1;
ALTER TABLE `{dbPrefix}fbi` AUTO_INCREMENT = 1;

CREATE TABLE IF NOT EXISTS `{dbPrefix}fbi_matrix` (
  `id` int(5) NOT NULL AUTO_INCREMENT,
  `subquestion_name` varchar(8) NOT NULL,
  `short_filename` varchar(255) NOT NULL,
  `email1` varchar(128) NOT NULL default '',
  `email2` varchar(128) NOT NULL default '',
  `sim` DECIMAL(5,2),
  PRIMARY KEY (`id`),
  UNIQUE KEY `fbi_matrix` (`short_filename`(50), `email1`(16), `email2`(16))
) AUTO_INCREMENT=1;
ALTER TABLE `{dbPrefix}fbi` AUTO_INCREMENT = 1;
