<?php

require ("q3_test_utils.php");
require ("q3.php");

// Run this script to test your code for Part A.1. It checks the return values of your function against test cases, and additionally print out a table showing all players

$tourMatchDAO = new TournamentMatchDAO();
// var_dump($tourMatchDAO );
$allTourMatches = $tourMatchDAO->getAllTournamentMatches();
$allPlayers = getAllPlayers($allTourMatches);

// Test cases
$playerTestCases = array(
    array("name" => "Brice Leverdez", "num_matches" => 66),
    array("name" => "Brian Yang", "num_matches" => 12),
    array("name" => "Lakshya Sen", "num_matches" => 45),
    array("name" => "Nan Wei", "num_matches" => 3),
);

$testCaseCounter = 0;
$testCasePassedCounter = 0;
foreach ($playerTestCases as $case) {
    // Check if the player can be found
    $playerFound = @$allPlayers[$case["name"]];
    if (isset ($playerFound)) {
        if (count($playerFound->getMatchesPlayed()) === $case["num_matches"]) {
            echo "Test case #$testCaseCounter: PASSED!<br>";
            $testCaseCounter++;
            $testCasePassedCounter++;
            continue;
        }
    }
    echo "Test case #$testCaseCounter: FAILED!<br>";
    $testCaseCounter++;
}
echo "Part A1: In total: $testCasePassedCounter / $testCaseCounter PASSED.";

$passA1 = $testCasePassedCounter;
$failA1 = $testCaseCounter - $testCasePassedCounter;
echo "<br>";

$allPlayers = calculatePlayerPerformanceStats($allPlayers);

// Test cases
$playerTestCases = array(
    array(
        "name" => "Long Chen",
        "num_matches_won" => 53,
        "best_perf" => "Final",
        "best_tour" => [
            "YONEX French Open 2019",
            "DANISA Denmark Open 2019",
            "CELCOM AXIATA Malaysia Open 2019",
            "PERODUA Malaysia Masters 2019",
            "YONEX French Open 2018",
        ]
    ),
    array(
        "name" => "Daren Liew",
        "num_matches_won" => 38,
        "best_perf" => "Semi final",
        "best_tour" => ["Barcelona Spain Masters 2020", "PERODUA Malaysia Masters 2018", "PRINCESS SIRIVANNAVARI Thailand Masters 2018"]
    ),
    array(
        "name" => "Jan O Jorgensen",
        "num_matches_won" => 34,
        "best_perf" => "Final",
        "best_tour" => ["YONEX Swiss Open 2018"]
    ),
    array(
        "name" => "Shiau Cheng Chen",
        "num_matches_won" => 3,
        "best_perf" => "Round of 32",
        "best_tour" => ["Lingshui China Masters 2019", "GWANGJU Korea Masters 2018"]
    ),
);

$testCaseCounter = 0;
$testCasePassedCounter = 0;
foreach ($playerTestCases as $case) {
    // Check if the player can be found
    $playerFound = @$allPlayers[$case["name"]];
    $num_won_passed = false;
    $best_perf_passed = false;
    $best_perf_tour_passed = false;
    if (isset ($playerFound)) {
        $num_won_passed = $playerFound->getNumMatchesWon() === $case["num_matches_won"];
        $best_perf_passed = $playerFound->getBestPerformanceRound()->getRoundStr() === $case['best_perf'];

        $playerBestPerfRounds = $playerFound->getBestPerformanceTournaments();
        $caseBestPerfRounds = $case['best_tour'];
        $best_perf_tour_passed = sort($playerBestPerfRounds) === sort($caseBestPerfRounds);

        if ($num_won_passed && $best_perf_passed && $best_perf_tour_passed) {
            echo "Test case #$testCaseCounter: PASSED!<br>";
            $testCaseCounter++;
            $testCasePassedCounter++;
            continue;
        }
    }
    echo "Test case #$testCaseCounter: FAILED!<br>";
    echo "Details: Passing number of matches won test-" . bool2str($num_won_passed) . " Passing best round of performance test-" . bool2str($best_perf_passed) . " Passing tournaments of best performance-" . bool2str($best_perf_tour_passed);
    echo "<br>";
    $testCaseCounter++;
}
echo "Part A2: In total $testCasePassedCounter / $testCaseCounter PASSED.";
$passA2 = $testCasePassedCounter;
$failA2 = $testCaseCounter - $testCasePassedCounter;

echo "<br>";

$h2hTestCases = array(
    array(
        "p1" => "Kenta Nishimoto",
        "p2" => "Rasmus Gemke",
        "win" => 3,
        "lose" => 2
    ),
    array(
        "p1" => "Daren Liew",
        "p2" => "Subhankar Dey",
        "win" => 1,
        "lose" => 0
    ),
    array(
        "p1" => "Chico Aura Dwi Wardoyo",
        "p2" => "Zii Jia Lee",
        "win" => 0,
        "lose" => 1
    ),
    array(
        "p1" => "Anthony Sinisuka Ginting",
        "p2" => "Long Chen",
        "win" => 5,
        "lose" => 2
    )
);

$testCaseCounter = 0;
$testCasePassedCounter = 0;
foreach ($h2hTestCases as $case) {
    // Check if the player can be found
    $h2hCase = getHeadToHeadStats($case["p1"], $case["p2"], $allPlayers);
    if (@$h2hCase["win"] === $case["win"] && @$h2hCase["lose"] === $case["lose"]) {
        echo "Test case #$testCaseCounter: PASSED!<br>";
        $testCaseCounter++;
        $testCasePassedCounter++;
        continue;
    }
    echo "Test case #$testCaseCounter: FAILED!<br>";
    $testCaseCounter++;
}
echo "Part B: In total: $testCasePassedCounter / $testCaseCounter PASSED.";
$passB = $testCasePassedCounter;
$failB = $testCaseCounter - $testCasePassedCounter;

echo "<br>";

$pass = $passA1 + $passA2 + $passB;
$fail = $failA1 + $failA2 + $failB;
echo "<h2>Summary</h2>";
echo "<h3 style='color:green'>Status: Passes $pass</h3>";
echo "<h4 style='color:green'>Q3A1: Passed $passA1. Q3A2: Passed $passA2. Q3B: Passed $passB.</h4>";
echo "<h3 style='color:red'>Status: Fails $fail</h3>";
echo "<h4 style='color:red'>Q3A1: Failed $failA1. Q3A2: Failed $failA2. Q3B: Failed $failB.</h4>";

$marks = ($passA1 + $passA2 + $passB) / 2;
echo "<h2>Marks</h2><h3 style='color:green'>Marks: $marks</h3>";
//  When staged from Manoj's Grader app, it looks for strings like 
// "question: marks" (e.g., q3: 6)
echo "<h3>q3: $marks</h3>"; 
